/*##CODE-DATEI
;-------------------------------------------------------------------------------
;##Copyright        :  ABB Automation Products GmbH
;
;-------------------------------------------------------------------------------
;##System           :  CONTRANS T
;##Systemkomponente :  Temperaturmessumformer TH02
;                      (Transmitter spez. Dokument: Rev. 1)
;-------------------------------------------------------------------------------
;##Programm         :  HC-DDL
;##Modul            :  TH02CO
;##Pfad/Datei       :  TH02CO.DDL
;##Library/Overlay  :  -/-
;##Programmierer    :  K.Pose / PLC-Alz
;##Erst.-Datum      :  02.03.2000
;##Version/Datum    :  01.00 / 02.03.2000
;-------------------------------------------------------------------------------
;##Beschreibung     :  Definition der privaten Kommandos und der dazugehoerenden
;                      Variablen.
;
;##SPECEND
;-------------------------------------------------------------------------------
;##*/


/* -------------------------------------------------------------------------- */
/*  VARIABLE messart:  Transmitter spezifische Tabelle IV MESSART             */
/*                                                                            */
/* -------------------------------------------------------------------------- */

VARIABLE messart
{
    LABEL [measurement_type];
    HELP  HLP_MESSART;

    CLASS DEVICE;

    TYPE ENUMERATED
    {
        IF( sensor_typ < 8 )
        {
                { 0x00, TXT_OHM50_EINF          },
                { 0x04, TXT_OHM60_EINF          },
                { 0x08, TXT_OHM50_DIFF          },
                { 0x18, TXT_OHM50_MITT          },
                { 0x0C, TXT_OHM60_DIFF          },
                { 0x1C, TXT_OHM60_MITT          }
        }
        IF( sensor_typ < 3 || sensor_typ > 7 )
        {
                { 0x40, TXT_MV50_EINF           },
                { 0x48, TXT_MV50_DIFF           },
                { 0x58, TXT_MV50_MITT           },
                { 0x44, TXT_MV60_EINF           },
                { 0x4C, TXT_MV60_DIFF           },
                { 0x5C, TXT_MV60_MITT           }
                IF( sensor_typ == 2 || sensor_typ > 8 )
                {
                        { 0x41, TXT_MV50_EINF_VGEXT     },
                        { 0x49, TXT_MV50_DIFF_VGEXT     },
                        { 0x59, TXT_MV50_MITT_VGEXT     },
                        { 0x43, TXT_MV50_EINF_VGINT     },
                        { 0x4B, TXT_MV50_DIFF_VGINT     },
                        { 0x5B, TXT_MV50_MITT_VGINT     },
                        { 0x45, TXT_MV60_EINF_VGEXT     },
                        { 0x4D, TXT_MV60_DIFF_VGEXT     },
                        { 0x5D, TXT_MV60_MITT_VGEXT     },
                        { 0x47, TXT_MV60_EINF_VGINT     },
                        { 0x4F, TXT_MV60_DIFF_VGINT     },
                        { 0x5F, TXT_MV60_MITT_VGINT     }
                }
        }
    }
    HANDLING READ & WRITE;
}


/* -------------------------------------------------------------------------- */
/*  VARIABLE sensor_typ:  Transmitter spezifische Tabelle VII  SENSOR_TYP     */
/* -------------------------------------------------------------------------- */

VARIABLE sensor_typ
{
    LABEL [sensor_type];
    HELP  HLP_SENSOR_TYP;

    CLASS CORRECTION;

    TYPE ENUMERATED
    {
       { 0,  TXT_LINEARLO   },
       { 1,  TXT_LINEARHI   },
       { 2,  TXT_FREISTIL   },
       { 3,  TXT_PT100      },
       { 4,  TXT_PT100_3916 },
       { 5,  TXT_PT100_3920 },
       { 6,  TXT_NI100      },
       { 7,  TXT_CU100      },
       { 8,  TXT_TYP_B      },
       { 9,  TXT_TYP_C      },
       { 10, TXT_TYP_D      },
       { 11, TXT_TYP_E      },
       { 12, TXT_TYP_J      },
       { 13, TXT_TYP_K      },
       { 14, TXT_TYP_L      },
       { 15, TXT_TYP_N      },
       { 16, TXT_TYP_R      },
       { 17, TXT_TYP_S      },
       { 18, TXT_TYP_T      },
       { 19, TXT_TYP_U      }
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE wire_no:  Transmitter spezifische Tabelle V                      */
/*                     ANZAHL DER SENSORANSCHLUSSLEITUNGEN                    */
/* -------------------------------------------------------------------------- */

VARIABLE wire_no
{
    LABEL  TXT_WIRE_NO;
    HELP   HLP_WIRE_NO;

    CLASS CORRECTION;

    TYPE ENUMERATED
    {
        { 2, [two_wires] }

        IF( !(messart & 0xC0) )
        {
            { 3, [three_wires] },
            { 4, [four_wires]  }
        }
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE n_sensor:  Transmitter spezifische Tabelle VI                    */
/*                      ANZAHL DER SENSOREN                                   */
/* -------------------------------------------------------------------------- */

VARIABLE n_sensor
{
    LABEL  TXT_N_SENSOR;
    HELP   HLP_N_SENSOR;

    CLASS  CORRECTION;

    TYPE ENUMERATED
    {
        IF( ( sensor_typ > 2 ) && ( sensor_typ < 8 ) )      /* wenn eingebaute RTDs */
        {
        
            /*
             *  bei eingebauten RTDs sind auch Teil-Sensoren sinnvoll
             *  (Pt10, Pt20, Pt50, Pt120)
             */

            { 0x12, TXT_18_SENSOR   },
            { 0x11, TXT_17_SENSOR   },
            { 0x10, TXT_HALB_SENSOR },
            { 0x13, TXT_19_SENSOR   }
        }

        { 0x01, TXT_01_SENSOR }

        IF( sensor_typ > 1 )        /* kein Linearer Sensortyp */
        {
            { 0x02, TXT_02_SENSOR },
            { 0x03, TXT_03_SENSOR },
            { 0x04, TXT_04_SENSOR },
            { 0x05, TXT_05_SENSOR },
            { 0x06, TXT_06_SENSOR },
            { 0x07, TXT_07_SENSOR },
            { 0x08, TXT_08_SENSOR },
            { 0x09, TXT_09_SENSOR },
            { 0x0A, TXT_10_SENSOR }
        }
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE sensor_monitor:  Transmitter spezifische Tabelle VIII            */
/*                            SENSOR-UEBERWACHUNG                             */
/* -------------------------------------------------------------------------- */

VARIABLE sensor_monitor
{
    LABEL  TXT_SENSOR_MONITOR;
    HELP   HLP_SENSOR_MONITOR;

    CLASS  CORRECTION;

    /* damit Kommando #48 etwas zu zeigen hat */

    TYPE ENUMERATED
    {
        { 0x00, TXT_NO_MONITOR          }
        IF( (messart & 0x18) == 0x00 )
        {
        /* einkanalig */
                { 0x01, TXT_EINGANG_BRUCH      }
                IF( !(messart & 0x40) )
                {
                /* R-Messung */
                        { 0x04, TXT_EINGANG_KURZ        },
                        { 0x05, TXT_EINGANG_BRUCH_KURZ  }
                }
        }
        ELSE
        {
        /* zweikanalig */
                { 0x03, TXT_ALL_BRUCH           }
                IF( !(messart & 0x40) )
                {
                /* R-Messung */
                        { 0x0C, TXT_ALL_KURZ            },
                        { 0x0F, TXT_ALL_BRUCH_KURZ      }
                }
        }

    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE refer_junction:  Temperatur der externen Vergleichsstelle        */
/* -------------------------------------------------------------------------- */

VARIABLE refer_junction
{
    LABEL  TXT_EXT_REF_TEMP;
    HELP   HLP_EXT_REF_TEMP;

    CLASS  CORRECTION;

    TYPE FLOAT
    {
        MIN_VALUE -40.0;
        MAX_VALUE 85.0;

        DISPLAY_FORMAT ".2f";       /* 2 Nachkommastellen,keine Mindestbreite */
        EDIT_FORMAT   "6.2f";
    }

    CONSTANT_UNIT [degC];

    HANDLING WRITE;
    VALIDITY IF(( messart & 0x03 ) == 0x01 )
             {
                TRUE;
             }
             ELSE
             {
                FALSE;
             }
}

/* -------------------------------------------------------------------------- */
/*  COMMAND schreiben_der_konfiguration: Kommando #131 schreibt die           */
/*                       Konfigurierung des Sensors zum Messumformer.          */
/* -------------------------------------------------------------------------- */

COMMAND schreiben_der_konfiguration
{
    NUMBER 131;

    OPERATION WRITE;

    TRANSACTION
    {
        REQUEST
        {
            messart,
            wire_no,
            sensor_typ,
            n_sensor,
            sensor_monitor,
            refer_junction
        }

        REPLY
        {
            response_code, device_status,

            messart,
            wire_no,
            sensor_typ,
            n_sensor,
            sensor_monitor,
            refer_junction
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,                 [no_command_specific_errors];
        5,  MISC_ERROR,              [too_few_data_bytes_recieved];
        6,  MODE_ERROR,              [xmtr_specific_command_error];
        7,  MODE_ERROR,              [in_write_protect_mode];
        9,  DATA_ENTRY_ERROR,        TXT_INVALID_MESSART;
        10, DATA_ENTRY_ERROR,        TXT_INVALID_N_SENSOR;
        11, DATA_ENTRY_ERROR,        [invalid_sensor_type_code];
        12, DATA_ENTRY_ERROR,        [invalid_sensor_connection_code];
        13, DATA_ENTRY_ERROR,        TXT_INVALID_MONITOR;
        28, MISC_WARNING,            TXT_REF_TEMP_TOO_LOW;
        29, MISC_WARNING,            TXT_REF_TEMP_TOO_HIGH;
    }
}



/*##KOMMANDO lesen_der_analog_ausgangs_parameter
;-------------------------------------------------------------------------------
;#Name             : lesen_der_analog_ausgangs_parameter
;
;#Dateiname        : TH02CO.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Lesen der Konfigurierung fuer den Analog-Ausgang.
;
;                    Das Kommando #132 liest die folgenden Variablen vom MU:
;
;                    - out_select
;                    - out_under
;                    - out_over
;                    - alter_selector
;                    - alter_value
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/


/* -------------------------------------------------------------------------- */
/*  VARIABLE out_select:  Transmitter spezifische Tabelle III                 */
/*                        AUSGANGS-SIGNALBEREICH                              */
/* -------------------------------------------------------------------------- */

VARIABLE out_select
{
    LABEL [outputs];
    HELP  HLP_OUT_SELECT;

    CLASS DEVICE;

    TYPE ENUMERATED
    {
        { 0x01, TXT_OUT_4_20MA }
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE out_under:  Untersteuerwert des Messumformers                     */
/* -------------------------------------------------------------------------- */

VARIABLE out_under
{
    LABEL TXT_OUT_UNDER;
    HELP  HLP_OUT_UNDER;

    CLASS CORRECTION;

    TYPE FLOAT
    {
        /*
         *  leider funktioniert hier die IF-ELSE-Struktur nicht, um den
         *  minimalen Untersteuerwert in Abhaengigkeit von der Schnittstelle
         *  (physical_signaling_code) einstellen zu koennen;
         *
         *  das HHT275 ist nur fuer FSK-Geraete vorgesehen => 3,6..4.0mA
         */

        MIN_VALUE 3.6;
        MAX_VALUE 4.0;

        DISPLAY_FORMAT ".2f";       /* 2 Nachkommastellen,keine Mindestbreite */
        EDIT_FORMAT   "6.2f";
    }

    CONSTANT_UNIT [mA];

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE out_over:  Uebersteuerwert des Messumformers                       */
/* -------------------------------------------------------------------------- */

VARIABLE out_over
{
    LABEL TXT_OUT_OVER;
    HELP  HLP_OUT_OVER;

    CLASS CORRECTION;

    TYPE FLOAT
    {
        MIN_VALUE 20.0;
        MAX_VALUE 23.6;

        DISPLAY_FORMAT ".2f";       /* 2 Nachkommastellen,keine Mindestbreite */
        EDIT_FORMAT   "6.2f";
    }

    CONSTANT_UNIT TXT_UNIT_MA;

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE alter_selector:  Transmitter spezifische Tabelle IX              */
/*                            AUSGANGS-VERHALTEN                              */
/*                                                                            */
/*                            Auswahl der Fehlerstrategie                     */
/* -------------------------------------------------------------------------- */

VARIABLE alter_selector
{
    LABEL TXT_ALTER_SELECTOR;
    HELP  HLP_ALTER_SELECTOR;

    CLASS CORRECTION;

    TYPE ENUMERATED
    {
        { 0x00, TXT_S_UEBER    },
        { 0x01, TXT_S_UNTER    },
        { 0x02, TXT_S_ERSATZ   },
        { 0x03, TXT_S_HALTEN   },
        { 0x10, TXT_SG_UEBER   },
        { 0x11, TXT_SG_UNTER   },
        { 0x12, TXT_SG_ERSATZ  },
        { 0x13, TXT_SG_HALTEN  },
        { 0xFB, TXT_SG_NOTHING }
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE alter_value:  Ersatzwert fuer den Fehlerfall                      */
/* -------------------------------------------------------------------------- */

VARIABLE alter_value
{
    LABEL TXT_ALTER_VALUE;
    HELP  HLP_ALTER_VALUE;

    CLASS CORRECTION;

    TYPE FLOAT
    {
        MIN_VALUE  -99.9;         
        MAX_VALUE  199.9;
    }

    CONSTANT_UNIT "%";

    VALIDITY IF( alter_selector==0x02 || alter_selector==0x12 )
             {
                 TRUE;
             }
             ELSE
             {
                 FALSE;
             }

    HANDLING READ & WRITE;
}

/* -------------------------------------------------------------------------- */
/*  COMMAND lesen_der_analog_ausgangs_parameter:                              */
/*                                                                            */
/*                 Die Parameter fuer den Analogausgang werden gelesen.        */
/* -------------------------------------------------------------------------- */

COMMAND lesen_der_analog_ausgangs_parameter
{
    NUMBER 132;

    OPERATION READ;

    TRANSACTION
    {
        REQUEST
        {}

        REPLY
        {
            response_code, device_status,

            out_select,
            out_under,
            out_over,
            alter_selector,
            alter_value
        }
    }

    RESPONSE_CODES
    {
        0, SUCCESS,                   [no_command_specific_errors];
        6, MODE_ERROR,                [xmtr_specific_command_error];
    }
}



/*##KOMMANDO schreiben_der_analog_ausgangs_parameter
;-------------------------------------------------------------------------------
;#Name             : schreiben_der_analog_ausgangs_parameter
;
;#Dateiname        : TH02CO.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Lesen der Konfigurierung fuer den Analog-Ausgang.
;
;                    Das Kommando #133 schreibt die folgenden Variablen in den
;                    Messumformer:
;
;                    - out_select
;                    - out_under
;                    - out_over
;                    - alter_selector
;                    - alter_value
;
;                    Die Definition der Variablen werden aus Kommando #132
;                    uebernommen.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

COMMAND schreiben_der_analog_ausgangs_parameter
{
    NUMBER 133;

    OPERATION WRITE;

    TRANSACTION
    {
        REQUEST
        {
            out_select,
            out_under,
            out_over,
            alter_selector,
            alter_value
        }

        REPLY
        {
            response_code, device_status,
            out_select,
            out_under,
            out_over,
            alter_selector,
            alter_value
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,                 [no_command_specific_errors];
        5,  MISC_ERROR,              [too_few_data_bytes_recieved];
        6,  MODE_ERROR,              [xmtr_specific_command_error];
        7,  MODE_ERROR,              [in_write_protect_mode];
        9,  DATA_ENTRY_ERROR,        [overrange_error];
        10, DATA_ENTRY_ERROR,        [lower_range_value_too_low];
        11, DATA_ENTRY_ERROR,        [upper_range_value_too_high];
        12, DATA_ENTRY_ERROR,        TXT_INV_ERROR_STRATEGIE;
        13, DATA_ENTRY_ERROR,        TXT_INV_ALTER_VALUE;
    }
}

/* -------------------------------------------------------------------------- */
/*  VARIABLE usr_threshold1:  Grenzwert 1                                     */
/* -------------------------------------------------------------------------- */

VARIABLE usr_threshold1
{
    LABEL TXT_THRESHOLD1;
    HELP  HLP_THRESHOLD;

    CLASS DISCRETE;

    TYPE FLOAT
    {
        MIN_VALUE  -6000.0;         
        MAX_VALUE   6000.0;
    }

    CONSTANT_UNIT "%";

    HANDLING READ & WRITE;
}

/* -------------------------------------------------------------------------- */
/*  VARIABLE usr_threshold2:  Grenzwert 2                                     */
/* -------------------------------------------------------------------------- */

VARIABLE usr_threshold2
{
    LABEL TXT_THRESHOLD2;
    HELP  HLP_THRESHOLD;

    CLASS DISCRETE;

    TYPE FLOAT
    {
        MIN_VALUE  -6000.0;         
        MAX_VALUE   6000.0;
    }

    CONSTANT_UNIT "%";

    HANDLING READ & WRITE;
}

/* -------------------------------------------------------------------------- */
/*  VARIABLE usr_threshold_status:  Staties der Grenzwerte                           */
/* -------------------------------------------------------------------------- */

VARIABLE usr_threshold_status
{
    HELP  HLP_THRESHOLD_STATUS;
    LABEL TXT_THRESHOLD_STATUS;

    CLASS DISCRETE & DIAGNOSTIC & DYNAMIC;

    TYPE BIT_ENUMERATED
    {
        { 0x01 ,  TXT_THRESHOLD1_OVER,  PROCESS },
        { 0x02 ,  TXT_THRESHOLD1_UNDER, PROCESS },
        { 0x04 ,  TXT_THRESHOLD2_OVER,  PROCESS },
        { 0x08 ,  TXT_THRESHOLD2_UNDER, PROCESS }
    }

    HANDLING READ;
}

/*##KOMMANDO lesen_der_grenzwerte
;-------------------------------------------------------------------------------
;#Name             : lesen_der_grenzwerte
;
;#Dateiname        : TH02CO.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Lesen der Grenzwerte. Signalisierung in Cmd#48,Byte 20
;
;                    Das Kommando #134 liest die folgenden Variablen:
;
;                    - usr_threshold1
;                    - usr_threshold2
;
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

COMMAND lesen_der_grenzwerte
{
    NUMBER 134;

    OPERATION READ;

    TRANSACTION
    {
        REQUEST
        { }

        REPLY
        {
            response_code, device_status,
            usr_threshold1,
            usr_threshold2,
            0
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,                 [no_command_specific_errors];
        6,  MODE_ERROR,              [xmtr_specific_command_error];
    }
}

/*##KOMMANDO schreiben_der_grenzwerte
;-------------------------------------------------------------------------------
;#Name             : schreiben_der_grenzwerte
;
;#Dateiname        : TH02CO.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Schreiben der Grenzwerte. Signalisierung in Cmd#48,Byte 20
;
;                    Das Kommando #135 liest die folgenden Variablen
;                    in den Messumformer:
;
;                    - usr_threshold1
;                    - usr_threshold2
;
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

COMMAND schreiben_der_grenzwerte
{
    NUMBER 135;

    OPERATION WRITE;

    TRANSACTION
    {
        REQUEST
        {
            usr_threshold1,
            usr_threshold2,
            0
        }

        REPLY
        {
            response_code, device_status,
            usr_threshold1,
            usr_threshold2,
            0
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,                 [no_command_specific_errors];
        5,  MISC_ERROR,              [too_few_data_bytes_recieved];
        6,  MODE_ERROR,              [xmtr_specific_command_error];
        7,  MODE_ERROR,              [in_write_protect_mode];
        9,  DATA_ENTRY_ERROR,        TXT_THRESHOLD1_RERROR;
        11, DATA_ENTRY_ERROR,        TXT_THRESHOLD2_RERROR;
    }
}

/*##KOMMANDO read_text
;-------------------------------------------------------------------------------
;#Name             : read_text
;
;#Dateiname        : TH02CO.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Lesen eines Info-Textes.
;
;                    Das Kommando #136 liest einen Infotext aus dem Messumformer.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

VARIABLE sensor_name
{
    LABEL  TXT_SENSOR_NAME;
    HELP   HLP_SENSOR_NAME;

    CLASS  CORRECTION;

    TYPE   PACKED_ASCII (20);
    HANDLING  READ & WRITE;
}


COMMAND read_text
{
    NUMBER 136;

    OPERATION READ;

    TRANSACTION 
    {
        REQUEST
        {
            1
        }

        REPLY
        {
            response_code, device_status,

            1,
            sensor_name
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,                  [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR,         [invalid_selection];
        6,  MODE_ERROR,               [xmtr_specific_command_error];
    }
}



/*##KOMMANDO write_text
;-------------------------------------------------------------------------------
;#Name             : write_text
;
;#Dateiname        : TH02CO.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Schreiben eines Info-Textes.
;
;                    Das Kommando #137 schreibt einen Infotext in den
;                    Messumformer.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

COMMAND write_text
{
    NUMBER 137;

    OPERATION WRITE;

    TRANSACTION 
    {
        REQUEST
        {
            1,
            sensor_name
        }

        REPLY
        {
            response_code, device_status,

            1,
            sensor_name
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,                  [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR,         [invalid_selection];
        5,  MISC_ERROR,               [too_few_data_bytes_recieved];
        6,  MODE_ERROR,               [xmtr_specific_command_error];
        7,  MODE_ERROR,               [in_write_protect_mode];
    }
}

/*##KOMMANDO eingangsjustierung_und_leitungsabgleich
;-------------------------------------------------------------------------------
;#Name             : eingangsjustierung_und_leitungsabgleich
;
;#Dateiname        : TH02CO.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Durchfuehrung von Leitungsabgleich und Eingangsjustage.
;
;                    Mit Kommando #163 kann der Leitungsabgleich und die
;                    Eingangsjustierung gestartet werden.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

VARIABLE kanal
{
    LABEL TXT_JUST_KANAL;
    HELP  HLP_JUST_KANAL;

    CLASS CORRECTION;

    TYPE  UNSIGNED_INTEGER;

    HANDLING WRITE;
}

VARIABLE sollwert
{
    LABEL [setpoint];
    HELP  HLP_JUST_SOLLWERT;

    CLASS CORRECTION;

    TYPE FLOAT
    {
        DISPLAY_FORMAT "8.3f";
        EDIT_FORMAT "8.3f";
    }
    HANDLING WRITE;
}



COMMAND eingangsjustierung_und_leitungsabgleich
{
    NUMBER 163;

    OPERATION COMMAND;

    TRANSACTION
    {
        REQUEST
        {
            kanal,
            trimin_unit,
            sollwert
        }

        REPLY
        {
            response_code, device_status,

            kanal,
            trimin_unit,
            sollwert
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,                  [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR,         [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR,         [passed_parameter_too_small];
        5,  MISC_ERROR,               [too_few_data_bytes_recieved];
        6,  MODE_ERROR,               [xmtr_specific_command_error];
        7,  MODE_ERROR,               [in_write_protect_mode];
        9,  DATA_ENTRY_ERROR,         TXT_INVALID_CHANNEL;
        10, DATA_ENTRY_ERROR,         TXT_INVALID_UNIT;
    }
}
