/*##CODE-DATEI
;-------------------------------------------------------------------------------
; ##Copyright        :  ABB Automation Products GmbH
;
;-------------------------------------------------------------------------------
; ##System           :  CONTRANS T
; ##Systemkomponente :  Temperaturmessumformer TH02
;                       (Transmitter spez. Dokument: Rev. 1)
;-------------------------------------------------------------------------------
; ##Programm         :  HC-DDL
; ##Modul            :  TH02ME
; ##Pfad/Datei       :  TH02ME.DDL
; ##Library/Overlay  :  -/-
; ##Programmierer    :  K.Pose / PLC-Alz
; ##Erst.-Datum      :  02.03.2000
; ##Version/Datum    :  01.00 / 02.03.2000
;-------------------------------------------------------------------------------
; ##Beschreibung     :  Definition der Menuestruktur zur Benutzerfuehrung im
;                       HHT275.
;
; ##SPECEND
;-------------------------------------------------------------------------------
;###*/



/*
 *  upload_variables:  Variablen fuer die OFFLINE-Konfigurierung.
 *
 *                     Leider ist die Plausibilisierung im Feldgeraet
 *                     so kompliziert, dass nur wenige Variablen hier
 *                     erscheinen duerfen.
 */

MENU upload_variables
{
    LABEL  TXT_UPLOAD_VARS;

    ITEMS
    {
        /* Messstellen-Daten */
        tag,
        descriptor,
        message,
        date,
        final_assembly_number,
        PV.SENSOR_SERIAL_NUMBER,

        /* Signalverarbeitung */
        PV.DAMPING_VALUE

    }
}

MENU configure_freestyle
{
    LABEL TXT_FREESTYLE_CONFIG;
    ITEMS
    {
        sensor_name,                         /* VARIABLE     */
        edit_curve                           /* METHOD       */
    }
}

MENU trimpreset_services
{
    LABEL TXT_TRIMPRESET_SERVICES;

    ITEMS
    {
                dac_trim,             /* method */
                scaled_dac_trim,      /* method */
                input_trim,           /* method */
                preset_mfgtrim,       /* method */
                preset_all            /* method */
    }
}

IMPORT STANDARD _MENU, DEVICE_REVISION 1, DD_REVISION 1
{
    EVERYTHING;                          /* zunaechst alle Menues importieren */

    REDEFINITIONS
    {
        /* einige Menues werden nicht benoetigt */

        DELETE MENU      status_display;
        DELETE MENU      construction_materials;
        DELETE MENU      menu_pres_sensor;
        DELETE MENU      frequency_output;
        DELETE MENU      discrete_output;

        DELETE VARIABLE  dummy_menu_entry;



        /*
         *  Hauptmenue muss ergaenzt werden...
         */

        MENU root_menu
        {
            REDEFINE ITEMS
            {
                device_setup,                                     /* MENU     */
                PV.DIGITAL_VALUE     (DISPLAY_VALUE),             /* VARIABLE */
                PV.ANALOG_VALUE      (DISPLAY_VALUE, READ_ONLY),  /* VARIABLE */
                PV.LOWER_RANGE_VALUE (DISPLAY_VALUE),             /* VARIABLE */
                PV.UPPER_RANGE_VALUE (DISPLAY_VALUE),             /* VARIABLE */
                _version                                          /* VARIABLE */
            }
        }



        /*
         *  Prozessvariablen:  Wichtige, aber einfache Einstelldaten werden
         *                     angezeigt.
         */

        MENU process_variables
        {
            REDEFINE ITEMS
            {
                primary_measurement.DIGITAL_VALUE     (DISPLAY_VALUE, READ_ONLY),
                primary_measurement.PERCENT_RANGE     (DISPLAY_VALUE, READ_ONLY),
                primary_measurement.ANALOG_VALUE      (DISPLAY_VALUE, READ_ONLY),
                sensor1_measurement_digital_value     (DISPLAY_VALUE, READ_ONLY),
                sensor2_measurement_digital_value     (DISPLAY_VALUE, READ_ONLY),
                electronics_temperature_digital_value (DISPLAY_VALUE, READ_ONLY)
            }
        }



        /*
         *  Diagnose/Service:  Hier werden Geraetepruef- und Abgleichroutinen
         *                     verwaltet.
         */

        MENU diag_service
        {
            REDEFINE ITEMS
            {
                test_device,          /* menu   */
                loop_test,            /* method */
                calibration,          /* menu   */
                wire_trim,            /* method */
                trimpreset_services   /* menu   */
            }
        }



        MENU test_device
        {
            /*
             *  Funktionen zur Feststellung und Darstellung des
             *  Geraetezustandes...
             */

            REDEFINE ITEMS
            {
                self_test,                           /* Selbsttest   (METHOD) */
                master_test                          /* Neustart     (METHOD) */
            }
        }



        MENU calibration
        {
            /*
             *  Funktionen fuer Einstellung und Abgleich im Bereich
             *  Service/Diagnose...
             */

            REDEFINE ITEMS
            {
                rerange,              /* Messbereich           (EDIT DISPLAY) */
                applied_rerange       /* Leitungsabgleich            (METHOD) */
            }
        }



        /*
         *  Grundeinstellung...
         */

        MENU basic_setup
        {
            REDEFINE ITEMS
            {
                rerange,                                      /* EDIT DISPLAY */
                PV.DAMPING_VALUE (DISPLAY_VALUE),             /* VARIABLE     */
                device_info                                   /* MENU         */
            }
        }



        /*
         *  Komplett-Setup...
         */

        MENU measuring_elements
        {
            /*
             *  Sensoren...
             */

            REDEFINE ITEMS
            {
                configure_sensor,                             /* METHOD       */
                configure_freestyle,                          /* MENU         */
                PV.SENSOR_SERIAL_NUMBER,                      /* VARIABLE     */
                menu_sensor_info,                             /* MENU         */
                wire_trim                                     /* METHOD       */
            }
        }


        MENU menu_sensor_info
        {
            /*
             *  Sensorinformationen...
             */

            REDEFINE ITEMS
            {
                PV.LOWER_SENSOR_LIMIT (DISPLAY_VALUE),
                PV.UPPER_SENSOR_LIMIT (DISPLAY_VALUE),
                PV.MINIMUM_SPAN       (DISPLAY_VALUE)
            }
        }



        MENU signal_conditioning
        {
            /*
             *  Signalverarbeitung...
             */

            REDEFINE ITEMS
            {
                primary_measurement.PERCENT_RANGE (DISPLAY_VALUE, READ_ONLY),
                primary_measurement.LOWER_RANGE_VALUE (DISPLAY_VALUE),
                primary_measurement.UPPER_RANGE_VALUE (DISPLAY_VALUE),
                select_unit,                                  /* METHOD       */
                primary_measurement.DAMPING_VALUE (DISPLAY_VALUE)
            }
        }



        MENU output_conditioning
        {
            /*
             *  Menue's fuer alle Ausgaenge:
             *
             *  - Analog-Ausgang
             *  - Messwertanzeiger
             *  - Digital-(HART)-Ausgang
             */

            REDEFINE ITEMS
            {
                analog_output,                                      /* MENUE  */
                hart_output                                         /* MENUE  */
            }
        }



        MENU analog_output
        {
            /*
             *  Analogausgang...
             */

            REDEFINE ITEMS
            {
                PV.ANALOG_VALUE (DISPLAY_VALUE, READ_ONLY),       /* VARIABLE */
                configure_output                                  /* METHOD   */
                loop_test,                                        /* METHOD   */
                dac_trim,                                         /* METHOD   */
                scaled_dac_trim                                   /* METHOD   */
            }
        }



        MENU hart_output
        {
            /*
             *  Einstellungen fuer die Kommunikation...
             */

            REDEFINE ITEMS
            {
                polling_address    (DISPLAY_VALUE),               /* Variable */
                response_preambles (DISPLAY_VALUE),               /* Variable */
                usr_threshold1,                                   /* Variable */
                usr_threshold2                                    /* Variable */
            }
        }



        MENU device_info
        {
            /*
             *  allgemeine Informationen ueber das angeschlossene Geraet und
             *  die dazugehoerende Messstelle...
             */

            REDEFINE ITEMS
            {
                tag(DISPLAY_VALUE),                               /* VARIABLE */
                descriptor(DISPLAY_VALUE),                        /* VARIABLE */
                message(DISPLAY_VALUE),                           /* VARIABLE */
                date(DISPLAY_VALUE),                              /* VARIABLE */
                final_assembly_number(DISPLAY_VALUE),             /* VARIABLE */

                private_label_distributor (DISPLAY_VALUE, READ_ONLY),
                device_type (DISPLAY_VALUE, READ_ONLY),
                write_protect (DISPLAY_VALUE, READ_ONLY),
                device_revisions                                  /* MENUE    */
            }
        }



        /*
         *  Ueberblick:  Die grosse Uebersicht ueber alle wichtigen (d.h.
         *               sinnvollen Variablen im Messumformer...
         *
         *               In der Reihenfolge sollten nicht Hersteller-Interessen,
         *               sondern Anwenderdaten im Vordergrund stehen.
         */

        MENU review
        {
            REDEFINE ITEMS
            {
                /* Messstellen-Daten */
                tag                       (DISPLAY_VALUE, READ_ONLY),
                descriptor                (DISPLAY_VALUE, READ_ONLY),
                message                   (DISPLAY_VALUE, READ_ONLY),
                date                      (DISPLAY_VALUE, READ_ONLY),
                final_assembly_number     (DISPLAY_VALUE, READ_ONLY),
                PV.SENSOR_SERIAL_NUMBER   (DISPLAY_VALUE, READ_ONLY),

                /* Signalverarbeitung */
                PV.RANGE_UNITS            (DISPLAY_VALUE, READ_ONLY),
                PV.LOWER_RANGE_VALUE      (DISPLAY_VALUE, READ_ONLY),
                PV.UPPER_RANGE_VALUE      (DISPLAY_VALUE, READ_ONLY),

                PV.LOWER_SENSOR_LIMIT     (DISPLAY_VALUE, READ_ONLY),
                PV.UPPER_SENSOR_LIMIT     (DISPLAY_VALUE, READ_ONLY),
                PV.MINIMUM_SPAN           (DISPLAY_VALUE, READ_ONLY),

                PV.DAMPING_VALUE          (DISPLAY_VALUE, READ_ONLY),

                /* Sensor-Konfigurierung */
                sensor_typ                (DISPLAY_VALUE, READ_ONLY),
                n_sensor                  (DISPLAY_VALUE, READ_ONLY),
                messart                   (DISPLAY_VALUE, READ_ONLY),
                wire_no                   (DISPLAY_VALUE, READ_ONLY),
                sensor_monitor            (DISPLAY_VALUE, READ_ONLY),

                /* Analog-Ausgang */
                out_select                (DISPLAY_VALUE, READ_ONLY),
                out_under                 (DISPLAY_VALUE, READ_ONLY),
                out_over                  (DISPLAY_VALUE, READ_ONLY),
                alter_selector            (DISPLAY_VALUE, READ_ONLY),
                alter_value               (DISPLAY_VALUE, READ_ONLY),


                /* HART-Ausgang */
                write_protect             (DISPLAY_VALUE, READ_ONLY),
                physical_signaling_code   (DISPLAY_VALUE, READ_ONLY),
                polling_address           (DISPLAY_VALUE, READ_ONLY),
                request_preambles         (DISPLAY_VALUE, READ_ONLY),

                /* Geraetebeschreibung */
                manufacturer_id           (DISPLAY_VALUE, READ_ONLY),
                private_label_distributor (DISPLAY_VALUE, READ_ONLY),
                device_type               (DISPLAY_VALUE, READ_ONLY),
                device_id                 (DISPLAY_VALUE, READ_ONLY),
                universal_revision        (DISPLAY_VALUE, READ_ONLY),
                transmitter_revision      (DISPLAY_VALUE, READ_ONLY),
                hardware_revision         (DISPLAY_VALUE, READ_ONLY),
                software_revision         (DISPLAY_VALUE, READ_ONLY)
            }
        }
    }
}


VARIABLE  _version
{
    LABEL    TXT_VERSION;
    CLASS    LOCAL;
    HANDLING READ;

    TYPE ENUMERATED
    {
        { 0, TXT_LANGUAGE, HLP_LANGUAGE }
    }
}
