/*##CODE-DATEI
;-------------------------------------------------------------------------------
;##Copyright        :  ABB Automation Products GmbH
;
;-------------------------------------------------------------------------------
;##System           :  CONTRANS T
;##Systemkomponente :  Temperaturmessumformer TH02
;                      (Transmitter spez. Dokument: Rev. 1)
;-------------------------------------------------------------------------------
;##Programm         :  HC-DDL
;##Modul            :  TH02MT
;##Pfad/Datei       :  TH02MT.DDL
;##Library/Overlay  :  -/-
;##Programmierer    :  K.Pose / PLC-Alz
;##Erst.-Datum      :  22.09.1999
;##Version/Datum    :  01.00 / 02.03.2000
;-------------------------------------------------------------------------------
;##Beschreibung     :  Definition der folgenden Methoden:
;
;                      - self_test
;                      - master_test
;                      - wire_trim
;                      - reset_parameter
;                      - configure_sensor
;                      - select_unit
;                      - test_range
;                      - configure_output
;                      - send_data
;
;##SPECEND
;-------------------------------------------------------------------------------
;###*/


#define MAXIMUM_SELFTEST_TIME           2     /* max. Anzahl der Warte-       */
                                              /* schleifen mit Kommando #48,  */
                                              /* nachdem der Selbsttest ge-   */
                                              /* startet wurde.               */
                                              /* Zusaetzlich sind noch 3s      */
                                              /* Wartezeit (Delay-Fkt.)       */
                                              /* noetig.                       */
                                              /*                              */
#define DEVICE_BUSY                     32    /* das Geraet ist beschaeftigt    */
                                              /* kann das Kommando nicht      */
                                              /* bearbeiten.                  */



/*##METHODE self_test
;-------------------------------------------------------------------------------
;#Name             : self_test
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Selbsttest
;
;                    Die Methode loest einen Selbsttest aus und meldet
;                    nacheinander alle ueber Kommando #48 gelieferten
;                    Unregelmaessigkeiten.
;
;                    Neben echten Fehlern sind dies auch Zustandsmeldungen sowie
;                    Unter- und Uebersteuerungen.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

METHOD self_test
{
    LABEL [selftest];
    HELP  HLP_SELF_TEST;

    DEFINITION
    {
        char status     [STATUS_SIZE];            /* STATUS_SIZE = 3          */
        char xmtr_status[MAX_XMTR_STATUS_LEN];    /* MAX_XMTR_STATUS_LEN = 25 */
        int  anzahl_der_versuche;

        IGNORE_ALL_DEVICE_STATUS();         /* ignoriere alle Bits des vom MU */
        XMTR_IGNORE_ALL_DEVICE_STATUS();    /* zurueckkommenden 2.Data Byte    */
                                            /* (Schreibzyklenzaehler im #48)   */
        IGNORE_ALL_RESPONSE_CODES();        /* ignoriere alle Bits des vom MU */
        XMTR_IGNORE_ALL_RESPONSE_CODES();   /* zurueckkommenden Response Code  */
                                            /* Bytes                          */

        ACKNOWLEDGE( TXT_MEASURE_STOP );    /* WARNUNG - Der Messbetrieb wird  */
                                            /*           unterbrochen.        */

        send_command(41);                   /* Selbsttest ausloesen            */
        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 41,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_SELFTEST );
            process_abort();
            /* ENDE */
        }

        DELAY( 3,TXT_SELFTEST_RUN );

        anzahl_der_versuche = 0;
        do
        {
            /* Kommando #48 sooft ausfuehren,     */
            /* bis der Selbsttest beendet ist... */
            get_more_status(status,xmtr_status);
        }
        while(    (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY  )
               && (anzahl_der_versuche++ < MAXIMUM_SELFTEST_TIME) );

        if( status[STATUS_RESPONSE_CODE] == DEVICE_BUSY )
        {
            /* Geraet ist im Selbsttest blockiert. */
            ACKNOWLEDGE( TXT_SELFTEST_BLOCKED );
            process_abort();
            /* ENDE */
        }
        else if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 48,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_SELFTEST );
            process_abort();
            /* ENDE */
        }
        else if(    xmtr_status[0]                 /* Speicherstatus          */
                 || xmtr_status[2]                 /* Eingangsstatus          */
                 || xmtr_status[3]                 /* Abgleich-/Konfi.-Status */
                 || xmtr_status[4]                 /* Sensorstatus            */
                 || xmtr_status[5] )               /* Ausgangsstatus          */
        {
            /*
             *  mind. eine Unregelmaessigkeit ist aufgetreten!
             *
             *  Es werden nicht alle Fehler gemeldet, um Folgeerscheinungen
             *  zunaechst einmal zu unterdruecken...
             */


            /* -------------------------------------------------------------- */
            /*  Speicherfehler:  Lese- oder Schreibfehler beim EEPROM werden  */
            /*                   einzeln nur gemeldet, wenn das Bauteil noch  */
            /*                   irgendwie ansprechbar ist, da INVALID_EEPROM */
            /*                   der Summenstatus ist.                        */
            /*                                                                */
            /*                   Was bei einem RAM- oder ROM-Fehler passiert, */
            /*                   steht sowieso in den Sternen...              */
            /* -------------------------------------------------------------- */

            if( INVALID_EEPROM & xmtr_status[0] )
            {
                /* nichts geht mehr... */
                display_xmtr_status( xmtr_specific_status_0,INVALID_EEPROM );
            }
            else
            {
                /* irgendwas funktioniert am EEPROM noch */
                if( EEPROM_WRITE      & xmtr_status[0] )
                    display_xmtr_status( xmtr_specific_status_0,EEPROM_WRITE );

                if( EEPROM_READ       & xmtr_status[0] )
                    display_xmtr_status( xmtr_specific_status_0,EEPROM_READ );

                if( EEPROM_WRITECYCLE & xmtr_status[0] )
                    display_xmtr_status( xmtr_specific_status_0,EEPROM_WRITECYCLE );
            }

            if( RAM_FAILURE  & xmtr_status[0] )
                display_xmtr_status( xmtr_specific_status_0,RAM_FAILURE );

            if( ROM_CHECKSUM & xmtr_status[0] )
                display_xmtr_status( xmtr_specific_status_0,ROM_CHECKSUM );


            /* -------------------------------------------------------------- */
            /*  ADC-Fehler:      Alle gemeldeten Fehler werden einzeln        */
            /*                   angezeigt, da sie unabhaengig voneinander     */
            /*                   auftreten koennen.                            */
            /* -------------------------------------------------------------- */

            if( ADC_NODATA & xmtr_status[2] )
                display_xmtr_status( xmtr_specific_status_2,ADC_NODATA );

            /* -------------------------------------------------------------- */
            /*  Abgleichfehler:  Hier werden nicht nur fehlerhafte Abgleich-  */
            /*                   versuche gemeldet, sondern auch der          */
            /*                   Betriebszustand (Abgleichmodus aktiv/nicht   */
            /*                   aktiv).                                      */
            /*                                                                */
            /*  Parameterstatus: Parameterfehler treten auf, wenn der MU      */
            /*                   nicht vollstaendig neu parametriert wurde.    */
            /*                   (Beispiel: Einheit der PV V_UNIT geaendert,   */
            /*                   aber Messbereich noch nicht korrigiert.       */
            /* -------------------------------------------------------------- */

            if( ABGLEICH_INAKTIV    & xmtr_status[3] )
                display_xmtr_status( xmtr_specific_status_3,ABGLEICH_INAKTIV );

            if( EINGANG_ABGLEICH    & xmtr_status[3] )
                display_xmtr_status( xmtr_specific_status_3,EINGANG_ABGLEICH );

            if( EINGANG1_JUSTIERUNG & xmtr_status[3] )
                display_xmtr_status( xmtr_specific_status_3,EINGANG1_JUSTIERUNG );

            if( EINGANG2_JUSTIERUNG & xmtr_status[3] )
                display_xmtr_status( xmtr_specific_status_3,EINGANG2_JUSTIERUNG );

            if( PARAMETER_FEHLER    & xmtr_status[3] )
                display_xmtr_status( xmtr_specific_status_3,PARAMETER_FEHLER );


            /* -------------------------------------------------------------- */
            /*  Sensorfehler:    Sensorfehler werden nur angezeigt, wenn die  */
            /*                   Eingangsschaltung (xmtr_status[2]) intakt    */
            /*                   ist und kein Speicherfehler vorliegt.        */
            /*                   In diesen Faellen ist naemlich der ganzen      */
            /*                   Messwertverarbeitung mehr zu trauen.          */
            /* -------------------------------------------------------------- */

            if(    !xmtr_status[0]                     /* kein Speicherfehler */
                && !xmtr_status[2] )                       /* kein ADC-Fehler */
            {
                if(      SENSOR1_BRUCH & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR1_BRUCH );
                else if( SENSOR1_KURZ  & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR1_KURZ  );
                else if( SENSOR1_UEBER & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR1_UEBER );
                else if( SENSOR1_UNTER & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR1_UNTER );

                if(      SENSOR2_BRUCH & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR2_BRUCH );
                else if( SENSOR2_KURZ  & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR2_KURZ  );
                else if( SENSOR2_UEBER & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR2_UEBER );
                else if( SENSOR2_UNTER & xmtr_status[4] )
                    display_xmtr_status( xmtr_specific_status_4,SENSOR2_UNTER );
            }


            /* -------------------------------------------------------------- */
            /*  Ausgangsstatus:  Der Status fuer den Analogausgang wird immer  */
            /*                   angezeigt, sofern etwas nicht in Ordnung     */
            /*                   ist.                                         */
            /* -------------------------------------------------------------- */

            if( AO_UNTERSTEUERT & xmtr_status[5] )
                display_xmtr_status( xmtr_specific_status_5,AO_UNTERSTEUERT );

            if( AO_UEBERSTEUERT & xmtr_status[5] )
                display_xmtr_status( xmtr_specific_status_5,AO_UEBERSTEUERT );

            if( AO_ERSATZWERT   & xmtr_status[5] )
                display_xmtr_status( xmtr_specific_status_5,AO_ERSATZWERT );

            if( AO_LETZTER_WERT & xmtr_status[5] )
                display_xmtr_status( xmtr_specific_status_5,AO_LETZTER_WERT );

            if( AO_FIXED        & xmtr_status[5] )
                display_xmtr_status( xmtr_specific_status_5,AO_FIXED );
        }
        else
        {
            /* -------------------------------------------------------------- */
            /*  es wurden keine Besonderheiten (Fehlfunktionen, Unter-,Ueber-  */
            /*  steuerunegn gefunden. Auf den eingebauten Messwertanzeiger muss */
            /*  allerdings selbst geachtet werden.                            */
            /* -------------------------------------------------------------- */

            ACKNOWLEDGE( TXT_SELFTEST_OK );
        }
    }
}



/*##METHODE master_test
;-------------------------------------------------------------------------------
;#Name             : master_test
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Messumformer-RESET incl. Selbsttest
;
;                    Die Methode loest einen Neustart (Reset) des Messumformers
;                    aus. Ein Selbsttest informiert ueber den anschliessenden
;                    Zustand des Geraetes.
;
;#Besonderheiten   : Auf eine einfache Warmstart-Funktion, die sich nur auf die
;                    Ausloesung von Kommando #42 beschraenkt, sollte wegen des
;                    hoeheren Informationsgehaltes dieser Methode verzichtet
;                    werden.
;
;-------------------------------------------------------------------------------
;##*/

METHOD master_test
{
    LABEL  TXT_MASTER_TEST;
    HELP   HLP_MASTER_TEST;

    DEFINITION
    {
        char status     [STATUS_SIZE];            /* STATUS_SIZE = 3          */
        char xmtr_status[MAX_XMTR_STATUS_LEN];    /* MAX_XMTR_STATUS_LEN = 25 */
        int  anzahl_der_versuche;

        IGNORE_ALL_DEVICE_STATUS();         /* ignoriere alle Bits des vom MU */
        XMTR_IGNORE_ALL_DEVICE_STATUS();    /* zurueckkommenden 2.Data Byte    */
                                            /* (Schreibzyklenzaehler im #48)   */
        IGNORE_ALL_RESPONSE_CODES();        /* ignoriere alle Bits des vom MU */
        XMTR_IGNORE_ALL_RESPONSE_CODES();   /* zurueckkommenden Response Code  */
                                            /* Bytes                          */

        /* ------------------------------------------------------------------ */
        /*  WARNUNG - Auch wenn es laestig ist, jetzt wird erst einmal vor den */
        /*            Folgen dieser Funktion gewarnt, also:                   */
        /*            - alle nicht netzausfallsicher gespeicherten Daten (ob  */
        /*              welche da sind oder nicht) gehen verloren.            */
        /*            - Regelautomatik auf Handbetrieb                        */
        /* ------------------------------------------------------------------ */

        ACKNOWLEDGE( TXT_RAM_DATA_LOSE );
        ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );

        add_abort_method( return_to_normal );

        send( 42,status );                              /* Warmstart ausloesen */
        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 42,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_MASTER_TEST );
            process_abort();
            /* ENDE */
        }

        DELAY( 3,TXT_RESET_WAIT );                      /* und 3s warten...   */

        anzahl_der_versuche = 0;
        do
        {
            /* Kommando #48 sooft ausfuehren,    */
            /* bis der Warmstart beendet ist... */
            get_more_status(status,xmtr_status);
        }
        while(    (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY  )
               && (anzahl_der_versuche++ < MAXIMUM_SELFTEST_TIME) );

        if( status[STATUS_RESPONSE_CODE] == DEVICE_BUSY )
        {
            /* Geraet ist im Neustart blockiert. */
            ACKNOWLEDGE( TXT_RESET_BLOCKED );
            process_abort();
            /* ENDE */
        }
        else if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 48,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_MASTER_TEST );
            process_abort();
            /* ENDE */
        }
        else
        {
            /* -------------------------------------------------------------- */
            /*  Der Warmstart ist erfolgreich verlaufen, aber leider fuehrt    */
            /*  Messumformer dabei keinen vollstaendigen Speichertest (wie      */
            /*  beim Selbsttest) durch.                                       */
            /*                                                                */
            /*  Daher wird der Selbsttest direkt hinterher geschickt und      */
            /*  erst anschliessend eine Auswertung der empfangenen Daten       */
            /*  durchgefuehrt.                                                 */
            /* -------------------------------------------------------------- */

            send_command(41);                 /* Selbsttest ausloesen          */
            if( status[STATUS_RESPONSE_CODE] )
            {
                /* der Messumformer wollte nicht... */
                display_response_status( 41,status[STATUS_RESPONSE_CODE] );
                ACKNOWLEDGE( TXT_NO_MASTER_TEST );
                process_abort();
                /* ENDE */
            }

            DELAY( 3,TXT_SELFTEST_RUN );      /* und 3 Sekunden warten...     */

            anzahl_der_versuche = 0;
            do
            {
                /* Kommando #48 sooft ausfuehren,     */
                /* bis der Selbsttest beendet ist... */
                get_more_status(status,xmtr_status);
            }
            while(    (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY  )
                   && (anzahl_der_versuche++ < MAXIMUM_SELFTEST_TIME) );

            if( status[STATUS_RESPONSE_CODE] == DEVICE_BUSY )
            {
                /* Geraet ist im Selbsttest blockiert. */
                ACKNOWLEDGE( TXT_SELFTEST_BLOCKED );
                process_abort();
                /* ENDE */
            }
            else if( status[STATUS_RESPONSE_CODE] )
            {
                /* der Messumformer wollte nicht... */
                display_response_status( 48,status[STATUS_RESPONSE_CODE] );
                ACKNOWLEDGE( TXT_NO_MASTER_TEST );
                process_abort();
                /* ENDE */
            }
            else if(    xmtr_status[0]             /* Speicherstatus          */
                     || xmtr_status[2]             /* Eingangsstatus          */
                     || xmtr_status[3]             /* Abgleich-/Konfi.-Status */
                     || xmtr_status[4]             /* Sensorstatus            */
                     || xmtr_status[5] )           /* Ausgangsstatus          */
            {
                /*
                 *  mind. eine Unregelmaessigkeit ist aufgetreten!
                 *
                 *  Es werden nicht alle Fehler gemeldet, um Folgeerscheinungen
                 *  zunaechst einmal zu unterdruecken...
                 */


                /* ---------------------------------------------------------- */
                /*  Speicherfehler:  Lese- oder Schreibfehler beim EEPROM     */
                /*                   werden einzeln nur gemeldet, wenn das    */
                /*                   Bauteil noch irgendwie ansprechbar ist,  */
                /*                   da INVALID_EEPROM der Summenstatus ist.  */
                /*                                                            */
                /*                   Was bei einem RAM- oder ROM-Fehler       */
                /*                   passiert, steht sowieso in den           */
                /*                   Sternen...                               */
                /* ---------------------------------------------------------- */

                if( INVALID_EEPROM & xmtr_status[0] )
                {
                    /* nichts geht mehr... */
                    display_xmtr_status( xmtr_specific_status_0,INVALID_EEPROM );
                }
                else
                {
                    /* irgendwas funktioniert am EEPROM noch */
                    if( EEPROM_WRITE      & xmtr_status[0] )
                        display_xmtr_status( xmtr_specific_status_0,EEPROM_WRITE );

                    if( EEPROM_READ       & xmtr_status[0] )
                        display_xmtr_status( xmtr_specific_status_0,EEPROM_READ );

                    if( EEPROM_WRITECYCLE & xmtr_status[0] )
                        display_xmtr_status( xmtr_specific_status_0,EEPROM_WRITECYCLE );
                }

                if( RAM_FAILURE  & xmtr_status[0] )
                    display_xmtr_status( xmtr_specific_status_0,RAM_FAILURE );

                if( ROM_CHECKSUM & xmtr_status[0] )
                    display_xmtr_status( xmtr_specific_status_0,ROM_CHECKSUM );


                /* ---------------------------------------------------------- */
                /*  ADC-Fehler:      Alle gemeldeten Fehler werden einzeln    */
                /*                   angezeigt, da sie unabhaengig             */
                /*                   voneinander auftreten koennen.            */
                /* ---------------------------------------------------------- */

                if( ADC_NODATA & xmtr_status[2] )
                    display_xmtr_status( xmtr_specific_status_2,ADC_NODATA );

                /* ---------------------------------------------------------- */
                /*  Abgleichfehler:  Hier werden nicht nur fehlerhafte        */
                /*                   Abgleichversuche gemeldet, sondern auch  */
                /*                   der Betriebszustand (Abgleichmodus       */
                /*                   aktiv/nicht aktiv).                      */
                /*                                                            */
                /*  Parameterstatus: Parameterfehler treten auf, wenn der MU  */
                /*                   nicht vollstaendig neu parametriert       */
                /*                   wurde. (Beispiel: Einheit der PV V_UNIT  */
                /*                   geaendert, aber Messbereich noch nicht     */
                /*                   korrigiert.                              */
                /* ---------------------------------------------------------- */

                if( ABGLEICH_INAKTIV      & xmtr_status[3] )
                    display_xmtr_status( xmtr_specific_status_3,ABGLEICH_INAKTIV );

                if( EINGANG_ABGLEICH     & xmtr_status[3] )
                    display_xmtr_status( xmtr_specific_status_3,EINGANG_ABGLEICH );

                if( EINGANG1_JUSTIERUNG   & xmtr_status[3] )
                    display_xmtr_status( xmtr_specific_status_3,EINGANG1_JUSTIERUNG );

                if( EINGANG2_JUSTIERUNG   & xmtr_status[3] )
                    display_xmtr_status( xmtr_specific_status_3,EINGANG2_JUSTIERUNG );

                if( PARAMETER_FEHLER & xmtr_status[3] )
                    display_xmtr_status( xmtr_specific_status_3,PARAMETER_FEHLER );


                /* ---------------------------------------------------------- */
                /*  Sensorfehler:    Sensorfehler werden nur angezeigt, wenn  */
                /*                   die Eingangsschaltung (xmtr_status[2])   */
                /*                   intakt ist und kein Speicherfehler       */
                /*                   vorliegt.                                */
                /*                   In diesen Faellen ist naemlich der ganzen  */
                /*                   Messwertverarbeitung mehr zu trauen.      */
                /* ---------------------------------------------------------- */

                if(    !xmtr_status[0]                 /* kein Speicherfehler */
                    && !xmtr_status[2] )                   /* kein ADC-Fehler */
                {
                    if(      SENSOR1_BRUCH & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR1_BRUCH );
                    else if( SENSOR1_KURZ  & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR1_KURZ  );
                    else if( SENSOR1_UEBER & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR1_UEBER );
                    else if( SENSOR1_UNTER & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR1_UNTER );

                    if(      SENSOR2_BRUCH & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR2_BRUCH );
                    else if( SENSOR2_KURZ  & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR2_KURZ  );
                    else if( SENSOR2_UEBER & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR2_UEBER );
                    else if( SENSOR2_UNTER & xmtr_status[4] )
                        display_xmtr_status( xmtr_specific_status_4,SENSOR2_UNTER );
                }


                /* ---------------------------------------------------------- */
                /*  Ausgangsstatus:  Der Status fuer den Analogausgang wird    */
                /*                   immer angezeigt, sofern etwas nicht in   */
                /*                   Ordnung ist.                             */
                /* ---------------------------------------------------------- */

                if( AO_UNTERSTEUERT & xmtr_status[5] )
                    display_xmtr_status( xmtr_specific_status_5,AO_UNTERSTEUERT );

                if( AO_UEBERSTEUERT & xmtr_status[5] )
                    display_xmtr_status( xmtr_specific_status_5,AO_UEBERSTEUERT );

                if( AO_ERSATZWERT   & xmtr_status[5] )
                    display_xmtr_status( xmtr_specific_status_5,AO_ERSATZWERT );

                if( AO_LETZTER_WERT & xmtr_status[5] )
                    display_xmtr_status( xmtr_specific_status_5,AO_LETZTER_WERT );

                if( AO_FIXED        & xmtr_status[5] )
                    display_xmtr_status( xmtr_specific_status_5,AO_FIXED );
            }
            else
            {
                /* ---------------------------------------------------------- */
                /*  es wurden keine Besonderheiten (Fehlfunktionen, Unter-,   */
                /*  Uebersteuerunegn gefunden. Auf den eingebauten Messwert-    */
                /*  anzeiger muss allerdings selbst geachtet werden.           */
                /* ---------------------------------------------------------- */

                ACKNOWLEDGE( TXT_SELFTEST_OK );

                process_abort();            /* damit die Regelschleife wieder */
                                            /* in Betrieb genommen wird.      */
            }
        }
    }       /*  Ende  Definition  */

}



/*##METHODE wire_trim
;-------------------------------------------------------------------------------
;#Name             : wire_trim
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Leitungsabgleich
;
;                    Die Methode fuehrt einen Leitungsabgleich durch. Dazu muss
;                    der Messumformer auf eine Widerstandsmessung in
;                    2-Leiterschaltung eingestellt sein.
;
;                    Ein Leitungsabgleich der internen Vergleichsstelle ist fuer
;                    den TH02 nicht vorgesehen.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

METHOD wire_trim
{
    LABEL TXT_WIRE_TRIM;
    HELP  HLP_WIRE_TRIM;

/*
    VALIDITY IF( (messart & 0x40)==0x00 && ( wire_no==2 ) )
             {
                 TRUE;
             }
             ELSE
             {
                 FALSE;
             }
*/

    DEFINITION
    {
        int   eingang;
        int   select;
        int   anzahl;
        int   counter;
        int   success;
        char  status     [STATUS_SIZE];
        char  xmtr_status[MAX_XMTR_STATUS_LEN];

        if( (ivar_value( messart ) & 0x40)==0 && ivar_value(wire_no)==2 )
        {
            /* Widerstandsmessung , 2-Leiter */

            IGNORE_ALL_DEVICE_STATUS();          /* diese Ueberwachungen werden    */
            IGNORE_ALL_RESPONSE_CODES();         /* in Eigenregie durchgefuehrt,   */
            XMTR_IGNORE_ALL_DEVICE_STATUS();     /* weil die Methode sonst bei    */
            XMTR_IGNORE_ALL_RESPONSE_CODES();    /* Leitungswiderstaenden < 0Ohm   */
                                                 /* abbricht.                     */

            ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );
            add_abort_method( return_to_normal );
    
            eingang = ivar_value( messart );

            /* Widerstandsmessung ist eingestellt */

            if( eingang & 0x08 )
            {
                /*
                 *  mehrkanalige Messung; daher muss der Kanal zunaechst
                 *  festgelegt werden...
                 */

                select = SELECT_FROM_LIST( TXT_WIRE_TRIM,TXT_LABGL_SELECT );
                ACKNOWLEDGE( TXT_LABGL_SHORT_ALL );
                ACKNOWLEDGE( TXT_LABGL_MAX_2 );
            }
            else
            {
                /*
                 *  bei einkanaliger Messung ist immer der erste Kanal,
                 *  d.h. der Eingang, gefragt...
                 */

                select = 0;

                /* 2-Leiter-Schaltung */
                ACKNOWLEDGE( TXT_LABGL_MAX_2 );
                ACKNOWLEDGE( TXT_LABGL_SHORT_INP );
            }
        }
        else
        {
            /*
             *  die Parametrierung erlaubt keinen Leitungsabgleich...
             */

            ACKNOWLEDGE( TXT_LABGL_NO_TRIM );
            process_abort();
        }

        DELAY( 5,TXT_LABGL_WAIT );            /* Wartezeit, damit sich der    */
                                              /* Eingangswert beruhigen       */
                                              /* kann...                      */

        iassign     ( kanal,select+3 );       /* festgelegter Kanal im MU...  */
        iassign     ( trimin_unit,37 );       /* die Einheit ist immer Ohm... */
        assign_float( sollwert,0.0 );         /* bei kurzgeschlossenem Sensor */
                                              /* sollte 0.0 gemessen werden.  */
                                              /*                              */
        send( 163,status );                   /* Leitungsabgleich starten...  */

        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 163,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_LABGL );
            process_abort();
            /* ENDE */
        }

        /* ------------------------------------------------------------------ */
        /*  Der Leitungsabgleich konnte gestartet werden; jetzt muss gefragt   */
        /*  werden bis zum bitteren Ende. Anschliessend koennen die Fehler      */
        /*  gemeldet werden, wenn welche aufgetreten sind.                    */
        /* ------------------------------------------------------------------ */

        do
        {
            get_more_status(status,xmtr_status);       /* #48 zum Messumformer */
        }
        while( xmtr_status[3] & 0x02 );                /* Abgleich noch aktiv */

        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 48,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_LABGL_NO_SUCCESS );
        }
        else
        {
            if(    (select == 0 && xmtr_status[3] & 0x04)
                || (select == 1 && xmtr_status[3] & 0x08) )
            {
                /* beim Abgleich ist ein Fehler aufgetreten... */
                ACKNOWLEDGE( TXT_LABGL_NO_SUCCESS );   /* melde Fehler und    */
                ACKNOWLEDGE( TXT_LABGL_TIP_1 );        /* gebe Tips...        */
            }
            else
            {
                /* ---------------------------------------------------------- */
                /*  Abgleich war erfolgreich, nun muessen die Daten noch       */
                /*  netzausfallsicher gespeichert werden, damit nicht alles   */
                /*  fuer die Katz' war...                                      */
                /* ---------------------------------------------------------- */

                DELAY( 2,TXT_EEPROM_WAIT );                /* Bitte warten... */

                iassign(eeprom_select,0);                   /* EEPROM brennen */
                send(39,status);

                if( status[STATUS_RESPONSE_CODE] )
                {
                    /* Daten konnten nicht gebrannt werden */
                    display_response_status(39,status[STATUS_RESPONSE_CODE]);
                    ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
                }
                else
                {
                    /*
                     *  bis hierher Standard, aber das RESET-Bit sollte noch
                     *  geloescht werden...
                     */

                    counter = 0;

                    do
                    {
                        /*
                         *  Kommando #48 ausfhren, bis eine
                         *  Antwort kommt...
                         */

                        send(48,status);
                    }
                    while( (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY) && (counter++ < 10) );

                    ACKNOWLEDGE( TXT_LABGL_SUCCESS );      /* melde Erfolg... */
                }
            }
        }

        process_abort();            /* Regelkreis wieder in Betrieb nehmen... */

    }       /* endDEF */

}       /* endMETHOD */



/*##METHODE reset_parameter
;-------------------------------------------------------------------------------
;#Name             : reset_parameter
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Zuruecksetzen der Parameter im RAM.
;
;                    Die Methode kopiert die EEPROM-Daten in das RAM und loescht
;                    damit alle nicht netzausfallsicher gespeicherten Daten.
;
;#Besonderheiten   : Die Methode ist als Abbruchfunktion gedacht.
;
;-------------------------------------------------------------------------------
;##*/

METHOD reset_parameter
{
    LABEL TXT_RESET_PARAMETER;
    HELP  HLP_RESET_PARAMETER;

    DEFINITION
    {
        char status[STATUS_SIZE];                          /* STATUS_SIZE = 3 */

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        /* Parameter werden zurueckgesetzt */
        PUT_MESSAGE( TXT_RESET_PARA_WAIT );

        iassign(eeprom_select,1);                            /* RAM neu laden */
        send(39,status);
        if( status[STATUS_RESPONSE_CODE] )
        {
            /* Daten konnten nicht gebrannt werden */
            display_response_status(39,status[STATUS_RESPONSE_CODE]);
            ACKNOWLEDGE( TXT_NO_PARA_RESET );
        }
        else
        {
            /*
             *  jetzt muessen die ganzen statischen Daten geladen werden,
             *  weil die REFRESH-Funktion (ueberraschend?) nicht
             *  funktioniert...
             */

            send(14,status);
            if( status[STATUS_RESPONSE_CODE] )
                display_response_status(14,status[STATUS_RESPONSE_CODE]);

            send(15,status);
            if( status[STATUS_RESPONSE_CODE] )
                display_response_status(15,status[STATUS_RESPONSE_CODE]);

            send(16,status);
            if( status[STATUS_RESPONSE_CODE] )
                display_response_status(16,status[STATUS_RESPONSE_CODE]);

            send(48,status);
            if( status[STATUS_RESPONSE_CODE] )
                display_response_status(48,status[STATUS_RESPONSE_CODE]);

            send(132,status);
            if( status[STATUS_RESPONSE_CODE] )
                display_response_status(132,status[STATUS_RESPONSE_CODE]);

        }
    }
}



/*##METHODE configure_sensor
;-------------------------------------------------------------------------------
;#Name             : configure_sensor
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Eingang/Sensor konfigurieren.
;
;                    Die Methode erlaubt dir Konfigurierung der zur Messwert-
;                    erfassung dienenden Variablen im Messumformer.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

VARIABLE _local_messart
{
    LABEL TXT_LOCAL_MESSART;
    HELP  HLP_LOCAL_MESSART;

    CLASS LOCAL;

    TYPE ENUMERATED
    {
        IF( sensor_typ < 3 )
        {
        /* Linear klein,gross oder Freistil */
                { 0x00,  TXT_OHM_EINF       },
                { 0x08,  TXT_OHM_DIFF       },
                { 0x18,  TXT_OHM_MITT       },
                { 0x40,  TXT_MV_EINF        },
                { 0x48,  TXT_MV_DIFF        },
                { 0x58,  TXT_MV_MITT        }
        }
        ELSE
        {
                IF( sensor_typ > 7 )
                {
                /* eingebaute TC */
                        { 0x40,  TXT_AL_EINF        },
                        { 0x48,  TXT_AL_DIFF        },
                        { 0x58,  TXT_AL_MITT        }
                }
                ELSE
                {
                /* eingebaute RTD */
                        { 0x00,  TXT_AL_EINF       },
                        { 0x08,  TXT_AL_DIFF       },
                        { 0x18,  TXT_AL_MITT       }
                }                        
        }
    }

    HANDLING WRITE;
}

VARIABLE _filter
{
    LABEL  TXT_FILTER;
    HELP   HLP_FILTER;

    CLASS  LOCAL;

    TYPE ENUMERATED
    {
        { 0x00, TXT_FILTER50  },
        { 0x04, TXT_FILTER60  }
    }
    HANDLING WRITE;
}

VARIABLE _type_of_refer_junc
{
    LABEL TXT_REF_JUNC_TYP;
    HELP  HLP_REF_JUNC_TYP;

    CLASS LOCAL;

    TYPE ENUMERATED
    {
            /* einkanalige Messung */
            IF( sensor_typ > 8 )
            {
                /* eingebaute TC-Kennlinien ohne Typ B... */
                { 0x03, TXT_REF_JUNC_INTERN },
                { 0x01, TXT_REF_JUNC_EXTERN },
                { 0x00, TXT_REF_JUNC_OHNE   }
            }
            ELSE
            {
                { 0x03, TXT_REF_JUNC_INTERN },
                { 0x01, TXT_REF_JUNC_EXTERN },
                { 0x00, TXT_REF_JUNC_OHNE   }
            }
    }

    HANDLING WRITE;
}



VARIABLE _sensor_monitor
{
    LABEL TXT_SENSOR_MONITOR;
    HELP  HLP_SENSOR_MONITOR;

    CLASS LOCAL;

    TYPE ENUMERATED
    {
        IF( !(messart & 0x40) )
        {
            /* Widerstandsmessung */
            { 0x00,  TXT_SENSOR_NO_MONITOR   },
            { 0x01,  TXT_SENSOR_BRUCH        },
            { 0x04,  TXT_SENSOR_KURZ         },
            { 0x05,  TXT_SENSOR_BRUCH_KURZ   }
        }
        ELSE
        {
            /* mV-Messung */
            { 0x00,  TXT_SENSOR_NO_MONITOR   },
            { 0x01,  TXT_SENSOR_BRUCH        }
        }
    }

    HANDLING WRITE;
}



METHOD configure_sensor
{
    LABEL TXT_CONFIG_SENSOR;
    HELP  HLP_CONFIG_SENSOR;

    DEFINITION
    {
        long ids[2];
        char status[STATUS_SIZE];            /* STATUS_SIZE = 3               */

        int    _messart;
        int    _unit;
        int    _value;
        float  _fvalue;
        float  _upper;
        float  _lower;

        IGNORE_ALL_DEVICE_STATUS();          /* Stoerungen werden vom HHT nur  */
        IGNORE_ALL_RESPONSE_CODES();         /* sehr primitiv abgefangen,     */
        XMTR_IGNORE_ALL_DEVICE_STATUS();     /* Umstellen auf eigene          */
                                             /* Meldungen.                    */

        /* ------------------------------------------------------------------ */
        /*  zunaechst muss eine angeschlossene Regelung auf Handbetrieb, sonst  */
        /*  kann's gewaltig krachen...                                        */
        /* ------------------------------------------------------------------ */

        ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );

        add_abort_method( return_to_normal );

        /* ------------------------------------------------------------------ */
        /*  jetzt ist es an der Zeit, alle notwendigen MU-Variablen in        */
        /*  lokalen Variablen zu sichern, um passende Anfangswerte zu er-     */
        /*  halten.                                                           */
        /* ------------------------------------------------------------------ */

        _messart = ivar_value( messart );
        _unit    = ivar_value( PV.RANGE_UNITS );

        /* ------------------------------------------------------------------ */
        /*  Die Temperatur fuer die externe Vergleichsstelle sollte fuer den  */
        /*  Fall, dass keine derartige Vergleichsstelle vorliegt, auf einen   */
        /*  Standardwert gesetzt werden.                                             */
        /* ------------------------------------------------------------------ */

        if( ivar_value(SV.DIGITAL_UNITS) == 250 )                 /* not used */
            assign_float( refer_junction, 20.0 );

        if( ( _messart & 0x03 ) == 0x01 )
        {
            assign_var( refer_junction, electronics_temperature_digital_value);
        }
        else
        {
            assign_float( refer_junction, 20.0 );
        }

        if(    (_messart & 0x40) == 0x40       /* mV */
            && (ivar_value(sensor_typ) == 2 || ivar_value(sensor_typ) > 8)
                                               /* Freistil oder TC ausser Typ B */
            && ivar_value(PV.RANGE_UNITS) != 253   /* keine exotische Einheit */ )
        {
            /* prinzipiell war eine Vergleichsstelle moeglich... */
            assign_int( _type_of_refer_junc,(_messart & 0x03) );
        }
        else
        {
            assign_int( _type_of_refer_junc, 0x03 );
        }
        /* ------------------------------------------------------------------ */
        /*  Sensortyp:  Alle eingebauten Kennlinien (incl. Linear und         */
        /*              Freistil) sind erlaubt. Damit sind u.U. schon einige  */
        /*              Daten vorgegeben und muessen nicht mehr eingegeben     */
        /*              werden.                                               */
        /* ------------------------------------------------------------------ */

        GET_DEV_VAR_VALUE( TXT_SENSOR_TYPE,sensor_typ );

        /* ------------------------------------------------------------------ */
        /*  eventl. sind einige Korrekturen notwendig, damit der Sensortyp    */
        /*  schon mal gesendet werden kann (wg. Plausibilisierung)...         */
        /*                                                                    */
        /*  Bei Linearer Kennlinie und Freistil ist (fast) alles erlaubt.     */
        /* ------------------------------------------------------------------ */

        if( 2 < ivar_value(sensor_typ) && ivar_value(sensor_typ) < 8 )
        {
        /* Eingebaute RTDs      */
            if( _messart & 0xE3 )         /* diese Bits haben bei Ohm-Messung */
                _messart &= ~0xE3;        /* nichts verloren...               */

            assign_float( refer_junction,20.0 );
        }
        else if( 7 < ivar_value(sensor_typ) )
        {
        /* Eingebaute TC */
            if( (_messart & 0xE0) != 0x40 )
            {
                _messart &= ~0xA0;
                _messart |=  0x40;

                if( ivar_value(sensor_typ)!=8 )                /* nicht Typ B */
                    _messart |= 0x01;
            }
        }
        else if( 2 > ivar_value(sensor_typ) )
        {
            /* Lineare Kennlinie */
            _messart &= ~0x03;
        }

        assign_int( messart,_messart );

        /* Anzahl der Sensoren korrigieren */
        if( _messart & 0x40 )
        {
            /* keine Ohm-, kOhm-Messung... */
            if( ivar_value(n_sensor) > 0x0A )
                assign_int(n_sensor,0x01);
        }

        /* Anzahl der Sensorleitungen */
        if( _messart & 0x48 )
            assign_int(wire_no,0x02);

        /* Sensor-Monitor aktualisieren */
        _value = ivar_value( sensor_monitor );

        if( _messart & 0x40 )
        {
            /*
             *  keine Ohm, kOhm-Messung -> kein Kurzschluss im 1. Kanal !
             *  wichtigster Test, weil Plausibilisierung im Messumformer
             */

            _value &= ~0x04;
        }

        _value &= ~0x0A;                                /* 2.Kanal loeschen */

        if( _messart & 0x08 )
        {
            /*
             *  mehrkanalige Messung -> 1.Kanal = 2.Kanal
             */
            _value |=  (_value << 1);                     /* 1.Kanal kopieren */
        }

        assign_int( sensor_monitor,_value );

        /* ------------------------------------------------------------------ */
        /*  was fuer ein Aufwand, damit in den Geraetevariablen die             */
        /*  Plausibilisierung neu durchgefuehrt werden kann !!!                */
        /* ------------------------------------------------------------------ */

        send( 131,status );          /* Daten uebertragen, damit die           */
                                     /* Plausibilisierung der Geraetevariablen */
                                     /* neu durchgefuehrt wird.                */

        if( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status( 131,status[STATUS_RESPONSE_CODE] );
            process_abort();
        }
        else
        {
            /*
             *  jetzt muessen mehrere Abbruchfunktionen in der richtigen
             *  Reihenfolge installiert werden...
             */

            remove_all_abort_methods();

            add_abort_method( reset_parameter );
            add_abort_method( return_to_normal );
        }

/* SENSORTYP fertig */

        /* ------------------------------------------------------------------ */
        /*  Eingangsbelegung:  Hier werden bereits Teile von 'messart' neu    */
        /*                     gesetzt. Bei Thermoelementen ist alles klar    */
        /*                     (mV-Messung), aber sonst muss nachgefragt      */
        /*                     werden.                                        */
        /*                                                                    */
        /*                                                                    */
        /*                     ACHTUNG - Auf eine gueltige Zusammenstellung   */
        /*                               der Bits in '_messart' sorgen,       */
        /*                               sonst kann der Wert nachher nicht    */
        /*                               nach 'messart' kopiert werden.       */
        /*                                                                    */
        /*                     Auch wenn die hier verwendete Eingangs-        */
        /*                     belegung eine LOKALE Groesse ist, muss die        */
        /*                     ueber die Buildin-Fkt. GET_DEV_VAR_VALUE(...)   */
        /*                     erfolgen, da '_input_range' innerhalb der DD   */
        /*                     global definiert ist.                          */
        /* ------------------------------------------------------------------ */



        /* ------------------------------------------------------------------ */
        /*  Messart:  Hier wird gewaehlt zwischen einfacher oder mehrkanaliger  */
        /*           Messung (sofern das ueberhaupt zulaessig ist).             */
        /*                                                                    */
        /* ------------------------------------------------------------------ */


        /* zunaechst muss die lokale Groesse korrekt initialisiert werden */

        assign_int( _local_messart,(_messart & 0x58) );

        GET_DEV_VAR_VALUE( TXT_LOCAL_MESSART,_local_messart );

        _messart &= ~0x58;    /* zuerst alles zurueck auf Einfach-Messung, */
                              /* damit definierte Anfangsbedingungen      */
                              /* geschaffen werden.                       */

        _messart |= ivar_value( _local_messart );

        if( !( _messart & 0x40) )
        {
        /* Ohm-Messung, also VG loeschen */
                _messart &= ~0x03;
        }

        /* ------------------------------------ */
        /* Netzspannungsfilter: 50 oder 60Hz    */
        /* ------------------------------------ */

        assign_int( _filter, (_messart & 0x04) );

        GET_DEV_VAR_VALUE( TXT_FILTER,_filter );

        _messart &= ~0x04;
        _messart |= ivar_value( _filter );

        assign_int( messart,_messart );

            /*
             *  und wieder muss plausibilisiert werden...
             */

        /* Anzahl der Sensoren korrigieren */
        if( _messart & 0x40 || (ivar_value(sensor_typ)==2) )
        {
            /* keine Ohm-, kOhm-Messung... */
            if( ivar_value(n_sensor) > 0x0A )
                assign_int(n_sensor,0x01);
        }

        /* Anzahl der Sensorleitungen */
        if( _messart & 0x48 )
            assign_int(wire_no,0x02);

        /* Sensor-Monitor aktualisieren */
        _value = ivar_value( sensor_monitor );

        if( _messart & 0x40 )
        {
            /*
             *  keine Ohm, kOhm-Messung -> kein Kurzschluss im 1. Kanal !
             *  wichtigster Test, weil Plausibilisierung im Messumformer
             */

            _value &= ~0x04;
        }

        _value &= ~0x0A;                                /* 2.Kanal loeschen */

        if( _messart & 0x08 )
        {
            /*
             *  mehrkanalige Messung -> 1.Kanal = 2.Kanal
             */
            _value |=  (_value << 1);                     /* 1.Kanal kopieren */
        }

        assign_int( sensor_monitor,_value );

            send( 131,status );             /* Daten uebertragen, damit die    */
                                            /* Plausibilisierung der Geraete-  */
                                            /* variablen neu durchgefuehrt     */
                                            /* wird.                          */


            if( status[STATUS_RESPONSE_CODE] )
            {
                display_response_status( 131,status[STATUS_RESPONSE_CODE] );
                process_abort();
            }
        

/* MESSART ausser VG fertig */
        /* ------------------------------------------------------------------ */
        /*  Einheit:  Moeglicherweise muss jetzt die Prozesseinheit geaendert     */
        /*            werden, weil der Sensor nicht mehr dazu passt.           */
        /* ------------------------------------------------------------------ */

        if( ivar_value(sensor_typ) > 1 )
        {
            /*
             *  KEINE lineare Kennlinie, also sind auch mehrere Einheiten
             *  moeglich...
             */

            if( 2<ivar_value(sensor_typ) )
            {
                /*
                 *  eingebaute Kennlinien arbeiten immer
                 *  temperaturlinear zum Ausgang.
                 *
                 *  Die Einheiten mV, Ohm und Sondereinheiten (=253)
                 *  sind verboten... => C
                 */

                if( _unit<32 || _unit>35 )
                {
                        assign_int( PV.RANGE_UNITS,32 );
                }
            }
            else
            {
                /* Freistil */
                if( (_unit<32 || _unit>35) && _unit!=253 )
                {
                    assign_int( PV.RANGE_UNITS,32 );
                }
            }
            GET_DEV_VAR_VALUE( TXT_PROCESS_UNIT,PV.RANGE_UNITS );
        }
        else
        {
            /*
             *  bei linearen Kennlinien werden auch die Einheiten 1:1
             *  durchgereicht...
             *
             *  die wichtigen Einstellungen ('_messart') sind mittlerweile
             *  bereinigt, also sind keine Dreckeffekte zu befuerchten...
             */

            if( _messart & 0x40 )
                assign_int( PV.RANGE_UNITS,36 );      /* mV  */
            else
                assign_int( PV.RANGE_UNITS,37 );      /* Ohm */
        }

        /*
         *  Einheit kann schon mal gesendet werden, weil die wichtigen Dinge
         *  dafuer schon gelaufen sind...
         */

        send( 44,status );
        if( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status( 44,status[STATUS_RESPONSE_CODE] );
            process_abort();
        }


        /* ------------------------------------------------------------------ */
        /*  Vergleichsstelle:  Nun ist die Vergleichsstelle an der Reihe,     */
        /*                     sofern das sinnvoll ist. Die Angaben teilen    */
        /*                     sich auf in Art der Vergleichsstelle (hier)    */
        /*                     und (bei externer Vergleichsstelle) die        */
        /*                     Temperatur jener (kommt spaeter).               */
        /* ------------------------------------------------------------------ */

        if(    (_messart & 0x40) == 0x40       /* mV */
            && (ivar_value(sensor_typ) == 2 || ivar_value(sensor_typ) > 8)
                                               /* Freistil oder TC ausser Typ B */
            && ivar_value(PV.RANGE_UNITS) != 253   /* keine exotische Einheit */ )
        {
            /*
             *  prinzipiell ist eine Vergleichsstelle nun moeglich...
             */

                 GET_DEV_VAR_VALUE( TXT_REF_JUNC_TYP,_type_of_refer_junc );

                 _messart &= ~0x03;          /* definierte Anfangsbedingungen */
                 _messart |= ivar_value(_type_of_refer_junc);   /* und setzen */

            /*
             *  Temperatur der externen Vergleichsstelle...
             */

            if( (_messart & 0x03) == 0x01 )
            {
                GET_DEV_VAR_VALUE( TXT_PROMPT_EXT_TEMP,refer_junction );
            }
            else
            {
                /* Standardwert */
                assign_float( refer_junction,20.0 );
            }
        }
        else
        {
            /*
             *  Vergleichsstelle nicht zulaessig, also kurzer Prozess...
             */

             _messart &= ~0x03;                     /* keine Vergleichsstelle */
                                                    /* Standardwert           */
             assign_float( refer_junction,20.0 );
        }

        /* ------------------------------------------------------------------ */
        /*  Sensorschaltung:  Nur bei einkanaliger Widerstandsmessung gibt    */
        /*                    es hier die freie Auswahl...                    */
        /* ------------------------------------------------------------------ */

        if( !(_messart & 0x48) )
        {
            /* einkanalige Widerstandsmessung */
            GET_DEV_VAR_VALUE( TXT_WIRE_NO, wire_no );
        }
        else
            assign_int( wire_no,2 );

        /*
         *  muss nicht sofort gesendet werden, weil keine weiteren Abhaengig-
         *  keiten vorhanden sind...
         */

        /* ------------------------------------------------------------------ */
        /*  Sensoranzahl:  Anzahl der Sensoren, eine einfache Uebung. Die     */
        /*                 Plausibilisierung (.1_.2_.5_1.2 nur bei RTDs)      */
        /*                 findet in 'n_sensor' statt.                        */
        /* ------------------------------------------------------------------ */

        if( ivar_value(sensor_typ) > 1 )
        {
        /* keine Lineare Kennlinie, also fragen */
                GET_DEV_VAR_VALUE( TXT_N_SENSOR,n_sensor );
        }
        else
        {
                assign_int( n_sensor, 0x01 );
        }

        /* ------------------------------------------------------------------ */
        /*  Sensorueberwachung...                                              */
        /*  Die Angabe haengt am ersten Kanal, weil der zweite nicht aktiv     */
        /*  oder genauso steht.                                               */
        /* ------------------------------------------------------------------ */

        assign_int( _sensor_monitor,(ivar_value(sensor_monitor) & 0x05) );

        GET_DEV_VAR_VALUE( TXT_SENSOR_MONITOR,_sensor_monitor );

        _value = ivar_value(_sensor_monitor);

        if( _messart & 0x08 )
        {
        /* 2-Kanal-Messung, also 1.Kanal kopieren */
                _value |=  (_value << 1);
        }

        assign_int( sensor_monitor, _value);

        /* ------------------------------------------------------------------ */
        /*   damit sind die Einstellungen erledigt, jetzt koennen die Daten    */
        /*   zum Geraet gesendet werden.                                       */
        /* ------------------------------------------------------------------ */

        /* Eingangsparameter (Kommando #131) */

        assign_int( messart,_messart );
        send( 131,status );
        if( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status( 131,status[STATUS_RESPONSE_CODE] );
            process_abort();
        }


        /* ------------------------------------------------------------------ */
        /*  Messbereich ggf. korrigieren...                                    */
        /* ------------------------------------------------------------------ */

        send( 15,status );                                /* Messbereich lesen */
        if( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status( 15,status[STATUS_RESPONSE_CODE] );
            process_abort();
        }

        send( 35,status );                     /* Messbereich wieder schreiben */
        if( status[STATUS_RESPONSE_CODE] )
        {
            if(    9 <= status[STATUS_RESPONSE_CODE]
                &&      status[STATUS_RESPONSE_CODE] <= 14 )
            {
                /* Messbereich passt nicht mehr */
                DELAY( 3,TXT_RANGE_CHANGED );

                send( 14,status );
                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status( 14,status[STATUS_RESPONSE_CODE] );
                    process_abort();
                }

                _lower  = fvar_value( PV.LOWER_SENSOR_LIMIT );
                _upper  = fvar_value( PV.UPPER_SENSOR_LIMIT );

                _fvalue = (_upper - _lower) / 100000.0;

                assign_float( PV.LOWER_RANGE_VALUE,_lower + _fvalue );
                assign_float( PV.UPPER_RANGE_VALUE,_upper - _fvalue );

                send( 35,status );          /* korrigierten Bereich schreiben */
                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status( 35,status[STATUS_RESPONSE_CODE] );
                    process_abort();
                }
            }
            else
            {
                display_response_status( 35,status[STATUS_RESPONSE_CODE] );
                process_abort();
            }
        }


        /* ------------------------------------------------------------------ */
        /*  und jetzt noch netzausfallsicher speichern...                     */
        /* ------------------------------------------------------------------ */

        remove_abort_method(reset_parameter);

        DELAY( 2,TXT_EEPROM_WAIT );                        /* Bitte warten... */

        iassign(eeprom_select,0);                           /* EEPROM brennen */
        send(39,status);

        if( status[STATUS_RESPONSE_CODE] )
        {
            /* Daten konnten nicht gebrannt werden */
            display_response_status(39,status[STATUS_RESPONSE_CODE]);
            ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
        }
        else
        {
            /*
             *  das mit der WHILE-Schleife verbraucht hier zuviel
             *  Speicherplatz, also mu getrickst werden...
             */

            send(48,status);
        }
    }

}



/*##METHODE select_unit
;-------------------------------------------------------------------------------
;#Name             : select_unit
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Prozesseinheit auswaehlen.
;
;                    Mit dieser Methode kann die Prozesseinheit umgestellt
;                    werden. Wegen der moeglicherweise notwendigen Hinweise ist
;                    hier eine Methode besser als das Handling mit PRE- und
;                    POST-EDIT-ACTIONS.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

METHOD select_unit
{
    LABEL TXT_PROCESS_UNIT;
    HELP  [digital_units_help];

    DEFINITION
    {
        int  old_unit;
        int  counter;
        int  unit;
        int  mail;

        float _fvalue;
        float _upper;
        float _lower;

        char status[STATUS_SIZE];                          /* STATUS_SIZE = 3 */

        mail = 0;      /* noch ist das Abchalten der Regelung nicht notwendig */

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        old_unit = ivar_value( PV.DIGITAL_UNITS );   /* alte Einheit sichern, */
                                                     /* um Wechsel ueberpruefen */
                                                     /* zu koennen.            */

        if( GET_DEV_VAR_VALUE( TXT_PROCESS_UNIT,PV.DIGITAL_UNITS ) == BI_SUCCESS )
        {
            /*
             *  Auswahl ohne Abbruch durchgefuehrt...
             */

            unit = ivar_value( PV.DIGITAL_UNITS );

            if(    old_unit != unit
                && (    !(old_unit==32 || old_unit==33 || old_unit==34 || old_unit==35)
                     || !(    unit==32 ||     unit==33 ||     unit==34 ||     unit==35) ) )
            {
                /*
                 *  Die Einheit hat sich geaendert und wenigstens eine der
                 *  Einheiten ist keine Temperatur-Einheit mehr...
                 */

                ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );
                add_abort_method( return_to_normal );

                mail = 1;     /* wenn die Einheit geaendert werden kann (#44), */
                              /* sollte der Messbereich ueberprueft werden,      */
                              /* bevor die Regelautomatik eingeschaltet wird. */
            }

            send( 44,status );
            if( status[STATUS_RESPONSE_CODE] )
            {
                display_response_status( 44,status[STATUS_RESPONSE_CODE] );
                process_abort();
            }

            if( mail )
            {
                remove_abort_method( return_to_normal );
                add_abort_method( test_range );

                /*
                 *  moeglicherweise muss der Messbereich noch in einen gueltigen
                 *  Bereich gesetzt werden...
                 */

                send( 15,status );                        /* Messbereich lesen */
                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status( 15,status[STATUS_RESPONSE_CODE] );
                    process_abort();
                }

                send( 35,status );             /* Messbereich wieder schreiben */
                if( status[STATUS_RESPONSE_CODE] )
                {
                    if(    9 <= status[STATUS_RESPONSE_CODE]
                        &&      status[STATUS_RESPONSE_CODE] <= 14 )
                    {
                        /* Messbereich passt nicht mehr */
                        ACKNOWLEDGE( TXT_RANGE_CHANGED );

                        send( 14,status );
                        if( status[STATUS_RESPONSE_CODE] )
                        {
                            display_response_status( 14,status[STATUS_RESPONSE_CODE] );
                            process_abort();
                        }

                        _lower  = fvar_value( PV.LOWER_SENSOR_LIMIT );
                        _upper  = fvar_value( PV.UPPER_SENSOR_LIMIT );

                        _fvalue = (_upper - _lower) / 100000.0;

                        assign_float( PV.LOWER_RANGE_VALUE,_lower + _fvalue );
                        assign_float( PV.UPPER_RANGE_VALUE,_upper - _fvalue );

                        send( 35,status );          /* korrigierten Bereich schreiben */
                        if( status[STATUS_RESPONSE_CODE] )
                        {
                            display_response_status( 35,status[STATUS_RESPONSE_CODE] );
                            process_abort();
                        }
                    }
                    else
                    {
                        display_response_status( 35,status[STATUS_RESPONSE_CODE] );
                        process_abort();
                    }
                }
            }

            /* -------------------------------------------------------------- */
            /*  nderungen netzausfallsicher speichern...                     */
            /* -------------------------------------------------------------- */

            DELAY( 2,TXT_EEPROM_WAIT );                    /* Bitte warten... */

            iassign(eeprom_select,0);                       /* EEPROM brennen */
            send(39,status);

            if( status[STATUS_RESPONSE_CODE] )
            {
                /* Daten konnten nicht gebrannt werden */
                display_response_status(39,status[STATUS_RESPONSE_CODE]);
                ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
            }
            else
            {
                /*
                 *  bis hierher Standard, aber das RESET-Bit sollte noch
                 *  geloescht werden...
                 */

                counter = 0;

                do
                {
                    /*
                     *  Kommando #48 ausfhren, bis eine
                     *  Antwort kommt...
                     */

                    send(48,status);
                }
                while( (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY) && (counter++ < 10) );
            }

            process_abort();
        }

    }   /* end DEFINITION */

}



/*##METHODE test_range
;-------------------------------------------------------------------------------
;#Name             : test_range
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Abbruchfunktion zu 'select_unit'.
;
;                    Die Methode weist darauf hin, dass der Messbereich vor dem
;                    dem Einschalten der Regelautomatik ueberprueft werden sollte.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

METHOD test_range
{
    LABEL [aborting_return_to_normal];

    DEFINITION
    {
        ACKNOWLEDGE( TXT_ACKN_TEST_RANGE );
    }
}



/*##METHODE configure_output
;-------------------------------------------------------------------------------
;#Name             : configure_output
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Ausgang konfigurieren.
;
;                    Die Methode erlaubt die Konfigurierung der zum Analog-
;                    ausgang gehoerenden Variablen im Messumformer.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

VARIABLE _failure_control
{
    LABEL TXT_FAILURE_CONTROL;
    HELP  HLP_FAILURE_CONTROL;

    CLASS LOCAL;

    TYPE ENUMERATED
    {
        IF( sensor_monitor )
        {
            { 0x00, TXT_SENSOR_FEHLER   },
            { 0x10, TXT_SENS_DEV_FEHLER },
            { 0xFB, TXT_NO_CONTROL      }
        }
        ELSE
        {
            { 0x10, TXT_DEV_FEHLER      },
            { 0xFB, TXT_NO_CONTROL      }
        }
    }
}



VARIABLE _failure_signal
{
    LABEL TXT_FAILURE_SIGNAL;
    HELP  HLP_FAILURE_SIGNAL;

    CLASS LOCAL;

    TYPE ENUMERATED
    {
        { 0x00, TXT_SIGNAL_HIGH  },
        { 0x01, TXT_SIGNAL_LOW   },
        { 0x02, TXT_SIGNAL_ALTER },
        { 0x03, TXT_SIGNAL_HOLD  }
    }
}



METHOD configure_output
{
    LABEL TXT_CONFIG_OUTPUT;
    HELP  HLP_CONFIG_OUTPUT;

    DEFINITION
    {
        int   counter;

        int   _ivalue;
        float _fvalue;
        char  status[STATUS_SIZE];

        IGNORE_ALL_DEVICE_STATUS();               /* sonst gibt es keine      */
        IGNORE_ALL_RESPONSE_CODES();              /* Moeglichkeit mehr, die    */
        XMTR_IGNORE_ALL_DEVICE_STATUS();          /* Fehler ordentlich zu     */
                                                  /* melden.                  */

        /* ------------------------------------------------------------------ */
        /*  Ausgangssignalbereich: Ganz einfach, weil bei 2-Leiter-MU nur     */
        /*                         4..20mA zulaessig sind.                     */
        /* ------------------------------------------------------------------ */

        iassign( out_select,0x01 );


        /* ------------------------------------------------------------------ */
        /*  Aussteuerbereich: Der Aussteuerbereich begrenzt den Analog-       */
        /*                    ausgang nicht nur im Fehlerfall, sondern auch   */
        /*                    Unter- oder Uebersteuerung des Messeingangs.      */
        /* ------------------------------------------------------------------ */

        if( ivar_value(physical_signaling_code)==2 )
            _fvalue = 3.0;
        else
            _fvalue = 3.6;

        do
        {
            GET_DEV_VAR_VALUE( TXT_INPUT_UNDER_VAL,out_under );

            /* gueltigen Bereich ueberpruefen */
            if( _fvalue <= float_value(out_under) && float_value(out_under) <= 4.0 )
                break;
        }
        while( 1 );

        do
        {
            GET_DEV_VAR_VALUE( TXT_INPUT_OVER_VAL,out_over );

            /* gueltigen Bereich ueberpruefen */
            if( 20.0 <= float_value(out_over) && float_value(out_over) <= 23.6 )
                break;
        }
        while( 1 );

        /* ------------------------------------------------------------------ */
        /*  Fehlersignalisierung: Welche Fehler sollen am Analogausgang       */
        /*                        gemeldet werden? Beim TH02 ist das der    */
        /*                        einzige Weg nach draussen...                 */
        /* ------------------------------------------------------------------ */

        _ivalue = ivar_value( alter_selector );

        if( _ivalue == 0xFB )                               /* initialisieren */
            assign_int( _failure_control,_ivalue );
        else
            assign_int( _failure_control,(_ivalue & 0x10) );

        /* auswaehlen */
        GET_DEV_VAR_VALUE( TXT_FAILURE_CONTROL,_failure_control );

        _ivalue = ivar_value( _failure_control );           /* setzen         */

        if( _ivalue == 0xFB )
            assign_var( alter_selector,_failure_control );
        else
        {
            _ivalue  = ivar_value( alter_selector );
            _ivalue &= ~0xFC;
            _ivalue |= ivar_value( _failure_control );
            assign_int( alter_selector,_ivalue );
        }


        /* ------------------------------------------------------------------ */
        /*  Ausgangssignal:  Wie soll der Ausgang auf einen oben              */
        /*                   eingestellten Fehler reagieren?                  */
        /* ------------------------------------------------------------------ */

        _ivalue = ivar_value(alter_selector);

        if( _ivalue != 0xFB )
        {
            /*
             *  es soll tatsaechlich eine Reaktion stattfinden...
             */

            assign_int( _failure_signal,_ivalue & 0x03 );

            GET_DEV_VAR_VALUE( TXT_FAILURE_SIGNAL,_failure_signal );

            _ivalue  = ivar_value( alter_selector );
            _ivalue &= ~0x03;
            _ivalue |= ivar_value( _failure_signal );
            assign_int( alter_selector,_ivalue );
        }


        /* ------------------------------------------------------------------ */
        /*  Ersatzwert: Der Ersatzwert erlaubt die Einstellung eines          */
        /*              variablen Fehlerwertes, der unabhaengig von den        */
        /*              eingestellten Aussteuergrenzen ist.                   */
        /*                                                                    */
        /*              Da der Wert in % angegeben wird, kann bei Nicht-      */
        /*              Benutzung eine Korrektur entfallen.                   */
        /* ------------------------------------------------------------------ */

        if( ivar_value(alter_selector) == 0x02 || ivar_value(alter_selector) == 0x12 )
        {
            /*
             *  Ersatzwert ist gewuenscht...
             */

            do
            {
                GET_DEV_VAR_VALUE( TXT_ALTER_VALUE,alter_value );

                /* gueltigen Bereich ueberpruefen */
                if(    -99.9 <= float_value(alter_value)
                    &&          float_value(alter_value) <= 199.9 )
                {
                    break;
                }
            }
            while( 1 );
        }
        else
        {
            /*
             *  sicherheitshalber sollte der Ersatzwert auf einen
             *  gueltigen Wert gesetzt werden...
             */

            assign_float( alter_value,100.0 );             /* Messbereichsende */
        }

        /* ------------------------------------------------------------------ */
        /*  Jetzt sind alle Daten zusammen und es kann geschrieben werden...  */
        /* ------------------------------------------------------------------ */

        send( 133,status );

        if( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status( 133,status[STATUS_RESPONSE_CODE] );
            process_abort();
        }


        /* ------------------------------------------------------------------ */
        /*  nderung jetzt noch netzausfallsicher speichern...                */
        /* ------------------------------------------------------------------ */

        DELAY( 2,TXT_EEPROM_WAIT );                        /* Bitte warten... */

        iassign(eeprom_select,0);                           /* EEPROM brennen */
        send(39,status);

        if( status[STATUS_RESPONSE_CODE] )
        {
            /* Daten konnten nicht gebrannt werden */
            display_response_status(39,status[STATUS_RESPONSE_CODE]);
            ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
        }
        else
        {
            /*
             *  bis hierher Standard, aber das RESET-Bit sollte noch
             *  geloescht werden...
             */

            counter = 0;

            do
            {
                /*
                 *  Kommando #48 ausfhren, bis eine
                 *  Antwort kommt...
                 */

                send(48,status);
            }
            while( (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY) && (counter++ < 10) );
        }

        process_abort();
    }

}



/*##METHODE send_data
;-------------------------------------------------------------------------------
;#Name             : send_data
;
;#Dateiname        : TH02MT.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Anmerkung ausgeben.
;
;                    Mit dieser Methode wird darauf hingewiesen, dass die
;                    nderungen erst nach Druecken der SENDE-Taste zum
;                    Messumformer uebertragen werden.
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##*/

METHOD send_data
{
    LABEL TXT_SEND_DATA;

    DEFINITION
    {
        ACKNOWLEDGE( TXT_PRESS_SEND_KEY );
    }
}

