/*##CODE-DATEI
;-------------------------------------------------------------------------------
;##Copyright        :  ABB Automation Products GmbH
;
;-------------------------------------------------------------------------------
;##System           :  CONTRANS T
;##Systemkomponente :  Temperaturmessumformer TH02
;                      (Transmitter spez. Dokument: Rev. 1)
;-------------------------------------------------------------------------------
;##Programm         :  HC-DDL
;##Modul            :  TH02R1
;##Pfad/Datei       :  TH02R1.DDL
;##Library/Overlay  :  -/-
;##Programmierer    :  K.Pose / PLC-Alz
;##Erst.-Datum      :  22.09.1999
;##Version/Datum    :  01.00 / 02.03.2000
;-------------------------------------------------------------------------------
;##Beschreibung     :  Basis zur Definition der Geraetebeschreibung fuer den
;                      Temperatur-Messumformer TH02.
;
;                      In dieser Datei erfolgt das Einbinden der weiteren
;                      notwendigen Beschreibungen sowie die Definition von
;                      ARRAY's, COLLECTION's, etc., wofuer die auch immer gut
;                      sein sollen.
;
;##SPECEND
;-------------------------------------------------------------------------------
;###*/


/* ************************************************************************** */

#ifndef __TH02__      /* ************************************************** */
#define __TH02__      /*   Hier wird die Geraetebeschreibung fuer den       */
#endif                /*   TH02 festgelegt.                                 */
                      /* ************************************************** */


#define __HC_275__      /* ************************************************** */
                        /*   Schaltet VARIABLEn, METHODs, etc. ab, die im     */
                        /*   HC275 zu Stoerungen oder unerwarteten            */
                        /*   Reaktionen fuehren.                              */
                        /*                                                    */
                        /*   Vielleicht koennen andere Bediengeraete diese    */
                        /*   Teile der Geraetebeschreibung benutzen.          */
                        /* ************************************************** */

/* ************************************************************************** */


#include "bi_codes.h"
#include "macros.ddl"



/* **************************************************************************** */

MANUFACTURER HARTMANN_BRAUN, DEVICE_TYPE _TH02, DEVICE_REVISION 1, DD_REVISION 1

/* **************************************************************************** */

/*
 *  Differenz-Messung gewaehlt...
 */

#define DIFF    ( (messart & 0x18) == 0x08 )


/*
 *  Mittelwert-Messung (mit/ohne Redundanz) gewaehlt...
 */

#define MID     ( (messart & 0x18) == 0x18 )



/*
 *  (un-)gueltige Vergleichsstelle
 */

#define JTMP    1


/*
 *  echte 2-Kanal-Messung (nicht mit Vergleichsstelle) aktiv ???
 */

#define VALID_MIDIFF       IF( DIFF || MID )   \
                           {                   \
                               TRUE;           \
                           }                   \
                           ELSE                \
                           {                   \
                               FALSE;          \
                           }


#define VALID_NON_MIDIFF   !VALID_MIDIFF


/*
 *  Vergleichsstellentemperatur gueltig ???
 */

#define VALID_JUNCTMP      IF( JTMP )          \
                           {                   \
                               TRUE;           \
                           }                   \
                           ELSE                \
                           {                   \
                               FALSE;          \
                           }

#define VALID_NON_JUNCTMP  !VALID_JUNCTMP


/*
 *  Die Reihenfolge der folgenden #include -Dateien muss eingehalten werden,
 *  damit Probleme beim Uebersetzen vermieden werden...
 */


#include "THxxx.ddl"

#include "TH02im.ddl"
#include "TH02co.ddl"
#include "TH02re.ddl"
#include "TH02mt.ddl"
#include "TH02fs.ddl"
#include "TH02tp.ddl"
#include "TH02me.ddl"


/*
 *  Dynamische (vom Messumformer selbstaetig aenderbare) Variablen
 *
 *  dahinter verbergen sich die wichtigsten Messwerte, naemlich
 *
 *  - Messwert (PV)
 *  - int./ext. Vergl.-Temperatur
 *  - Messwerte der einzelnen Sensoren (in elektr. Einheit)
 */

ARRAY OF COLLECTION dynamic_variables
{
    LABEL TXT_DYNAMIC_VARIABLES;
    HELP [dynamic_variable_help];

    ELEMENTS
    {
        PRIMARY,    transmitter_variables[0],  [pv],  TXT_PRIMARY_MEASURE;
        SECONDARY,  transmitter_variables[1],  [sv],  TXT_SECONDARY_MEASURE;
        TERTIARY,   transmitter_variables[2],  [tv],  [sensor_a_temperature_measurement_value];
        FOURTH,     transmitter_variables[3],  [qv],  [sensor_b_temperature_measurement_value];
    }
}



/*
 *  Zusammenfassung der Feldgeraete-Variablen...
 */

ARRAY OF COLLECTION transmitter_variables
{
    LABEL [transmitter_variable];
    HELP  [transmitter_variable_help];

    ELEMENTS
    {
        0, primary_measurement,            TXT_PRIMARY_MEASURE,   HLP_PRIMARY_MEASURE;
        1, reference_junction_temperature, TXT_SECONDARY_MEASURE, HLP_SECONDARY_MEASURE;
        2, sensor1_measurement,            [sensor_a_temperature_measurement_value];
        3, sensor2_measurement,            [sensor_b_temperature_measurement_value];
    }
}



/* ************************************************************************** */
/*                                                                            */
/*                       PV - H A U P T V A R I A B L E                       */
/*                                                                            */
/* ************************************************************************** */

COLLECTION OF VARIABLE primary_measurement
{
    HELP    IF MID
            {
                HLP_PM_COLLECT_MID;
            }
            ELSE
            {
                IF DIFF
                {
                    HLP_PM_COLLECT_DIFF;
                }
                ELSE
                {
                    HLP_PM_COLLECT_SINGLE;
                }
            }

    LABEL   IF MID
            {
                TXT_PM_COLLECT_DIFF;
            }
            ELSE
            {
                IF DIFF
                {
                    [differential_measurement];
                }
                ELSE
                {
                    TXT_PM_COLLECT_SINGLE;
                }
            }

    MEMBERS
    {

        DIGITAL_UNITS,         digital_units,                           [units],                 [digital_units_help];
        DIGITAL_VALUE,         primary_measurement_digital_value,       TXT_PV_VALUE,            [digital_value_help];
        PERCENT_RANGE,         primary_measurement_percent_range,       TXT_PV_PERCENT,          [percent_of_range_help];

        TRANSFER_FUNCTION,     analog_output_transfer_function,         [transfer_function],     [transfer_function_help];

        RANGE_UNITS,           digital_units,                           [range_units],           [range_units_help];
        UPPER_RANGE_VALUE,     primary_measurement_upper_range_value,   [urv],                   [upper_range_value_help];
        LOWER_RANGE_VALUE,     primary_measurement_lower_range_value,   [lrv],                   [lower_range_value_help];
        DAMPING_VALUE,         primary_measurement_damping_value,       [damping_value],         [seconds_damping_value_help];

        SENSOR_SERIAL_NUMBER,  sensor_serial_number,                    [sensor_serial_number],  [sensor_serial_number_help];
        SENSOR_UNITS,          digital_units,                           [sensor_unit],           [sensor_minspan_unit_help];
        UPPER_SENSOR_LIMIT,    primary_measurement_upper_sensor_limit,  [usl],                   [upper_sensor_limit_help];
        LOWER_SENSOR_LIMIT,    primary_measurement_lower_sensor_limit,  [lsl],                   [lower_sensor_limit_help];
        MINIMUM_SPAN,          primary_measurement_minimum_span,        [minimum_span],          [minimum_span_help];

        ANALOG_VALUE,          ma_analog_output_value,                  [analog_output_value],   [analog_value_help];
        ALARM_CODE,            ma_analog_output_alarm_select,           [alarm_select],          [alarm_code_help];

    }
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE digital_units:  Common - Tables II  UNIT_CODES                   */
/*                                                                            */
/*                           Diese Variable ist fuer die Einheiten gedacht,   */
/*                           die mit den Messwerten, Messbereichen, Sensor-   */
/*                           grenzen, etc. gelesen und geschrieben werden.    */
/*                                                                            */
/*                           Um Refresh-Probleme zu vermeiden, wird die       */
/*                           Variable einfach als dynamisch vereinbart.       */
/* -------------------------------------------------------------------------- */

VARIABLE digital_units
{
    LABEL [units];
    HELP  [digital_units_help];

    CLASS INPUT;

    TYPE ENUMERATED
    {
        IF( sensor_typ < 2 )                       /* linear 1:1             */
        {
            IF( messart & 0x40 )
            {
                UNITS_CODE (36)                     /* millivolts             */
            }
            ELSE
            {
                UNITS_CODE (37)                     /* ohms                   */
            }
        }
        ELSE
        {
            IF( sensor_typ == 2 )                   /* Freistil               */
            {
                /* alles zulaessig (im Rahmen) */
                UNITS_CODE (32),                    /* degrees Celsius        */
                UNITS_CODE (33),                    /* degrees Fahrenheit     */
                UNITS_CODE (34),                    /* degrees Rankine        */
                UNITS_CODE (35),                    /* Kelvin                 */
                UNITS_CODE (253)                    /* -/-                    */
            }
            ELSE
            {
                /* RTD oder TC */
                UNITS_CODE (32),                /* degrees Celsius        */
                UNITS_CODE (33),                /* degrees Fahrenheit     */
                UNITS_CODE (34),                /* degrees Rankine        */
                UNITS_CODE (35)                 /* Kelvin                 */
            }
        }
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_digital_value:  PV - Messwert; bei zwei-     */
/*                                               kanaliger Messung nach der   */
/*                                               Verknuepfung der Kanaele.    */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_digital_value
{
    HELP    HLP_PV_VALUE;
    LABEL   IF MID
            {
                TXT_PV_MID_VALUE;
            }
            ELSE
            {
                IF DIFF
                {
                    TXT_PV_DIFF_VALUE;
                }
                ELSE
                {
                    TXT_PV_VALUE;
                }
            }

    CLASS INPUT & DYNAMIC;

    TYPE  FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }

    HANDLING READ;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_percent_range:  WMAT - Prozentwert der durch */
/*                                               den parametrierten Mess-     */
/*                                               bereich gegebenen Ausgangs-  */
/*                                               spanne.                      */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_percent_range
{
    HELP    HLP_PERCENT_RANGE;
    LABEL   IF MID
            {
                TXT_PV_MID_VALUE;
            }
            ELSE
            {
                IF DIFF
                {
                    TXT_PV_DIFF_VALUE;
                }
                ELSE
                {
                    TXT_PV_VALUE;
                }
            }

    CLASS INPUT & DYNAMIC;

    TYPE FLOAT
    {
        EDIT_FORMAT    "2.2f";
        DISPLAY_FORMAT  ".2f";
    }

    CONSTANT_UNIT "%";

    HANDLING READ;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE analog_output_transfer_function:  Gibt Hinweise auf die          */
/*                                             Messwertverarbeitung im        */
/*                                             Feldgeraet.                    */
/*                                                                            */
/*                LINEAR wird bei eingebauten Kennlinien oder ausgeschalteter */
/*                Linearisierung angezeigt. Bei Freistilkennlinien wird dann  */
/*                SPECIAL angezeigt.                                          */
/* -------------------------------------------------------------------------- */

VARIABLE analog_output_transfer_function
{
    LABEL [transfer_function];
    HELP  [transfer_function_help];

    TYPE ENUMERATED
    {
        TRANSFER_FUNCTION_CODE   (0)     /* Linear                 */
        TRANSFER_FUNCTION_CODE (253)     /* spezial, Freistilkurve */
    }

    CLASS INPUT;
    HANDLING READ;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_upper_range_value:                           */
/*                                                                            */
/*                Prozesssignal, welches dem 20mA-Ausgangssignal entspricht.  */
/*                Eine fallende Kennlinie erhaelt man, wenn gilt:             */
/*                                                                            */
/*                primary_measurement_lower_range_value >                     */
/*                                     primary_measurement_upper_range_value  */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_upper_range_value
{
    HELP    HLP_UPPER_RANGE_VALUE;
    LABEL   [urv];

    CLASS INPUT;

    TYPE FLOAT
    {
        DISPLAY_FORMAT  ".2f";
        EDIT_FORMAT    "6.2f";
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_upper_range_value:                           */
/*                                                                            */
/*                Prozesssignal, welches dem 4mA-Ausgangssignal entspricht.   */
/*                Eine fallende Kennlinie erhaelt man, wenn gilt:             */
/*                                                                            */
/*                primary_measurement_lower_range_value >                     */
/*                                     primary_measurement_upper_range_value  */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_lower_range_value
{
    HELP    HLP_LOWER_RANGE_VALUE;
    LABEL   [lrv];

    CLASS INPUT;

    TYPE FLOAT
    {
        DISPLAY_FORMAT  ".2f";
        EDIT_FORMAT    "6.2f";
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_damping_value:                               */
/*                                                                            */
/*                Zeitkonstante des PT1-Daempfungsfilters im Feldgeraet.      */
/*                                                                            */
/*                Die Zeitkonstante 0.0s fuehrt zum Abschalten des Filters    */
/*                und der zusaetzlichen 50Hz-Unterdrueckung.                  */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_damping_value
{
    HELP    HLP_DAMPING_VALUE;
    LABEL   [damping_value];

    CLASS INPUT;

    TYPE FLOAT
    {
        DISPLAY_FORMAT  ".1f";
        EDIT_FORMAT    "4.1f";

        MIN_VALUE1     0.0;                        /* Daempfung ausgeschaltet */
        MAX_VALUE1   100.0;

    }

    CONSTANT_UNIT  TXT_UNIT_SEK;

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE sensor_serial_number:  Sensor-Seriennummer; ohne Einfluss auf    */
/*                                  die Messwert-Verarbeitung.                */
/* -------------------------------------------------------------------------- */

VARIABLE sensor_serial_number
{
    HELP  [sensor_serial_number_help];
    LABEL [sensor_serial_number];

    CLASS DEVICE;

    TYPE UNSIGNED_INTEGER (3)
    {
       MIN_VALUE          0;
       MAX_VALUE   16777215;
       DISPLAY_FORMAT  "8u";
       EDIT_FORMAT     "8u";
    }

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_upper_sensor_limit:                          */
/*                                                                            */
/*                Obere Einstellgrenze fuer den Messbereich. Bei 2-kanaligen  */
/*                Messungen ist das nicht der obere Sensorbereich, sondern    */
/*                die nach der Verknuepfung der Kanaele moegliche Grenze.     */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_upper_sensor_limit
{
    HELP    HLP_UPPER_SENSOR_LIMIT;
    LABEL   [usl];

    CLASS CORRECTION;

    TYPE FLOAT
    {
        DISPLAY_FORMAT  ".2f";
        EDIT_FORMAT    "6.2f";
    }

    HANDLING READ;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_lower_sensor_limit:                          */
/*                                                                            */
/*                Untere Einstellgrenze fuer den Messbereich. Bei 2-kanaligen */
/*                Messungen ist das nicht der untere Sensorbereich, sondern   */
/*                die nach der Verknuepfung der Kanaele moegliche Grenze.     */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_lower_sensor_limit
{
    HELP    HLP_LOWER_SENSOR_LIMIT;
    LABEL   [lsl];

    CLASS CORRECTION;

    TYPE FLOAT
    {
        DISPLAY_FORMAT  ".2f";
        EDIT_FORMAT    "6.2f";
    }

    HANDLING READ;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE primary_measurement_minimum_span:                                */
/*                                                                            */
/*                Minimale Spanne fuer den Messbereich.                       */
/* -------------------------------------------------------------------------- */

VARIABLE primary_measurement_minimum_span
{
    HELP    HLP_MINIMUM_SPAN;
    LABEL   [minimum_span];

    CLASS CORRECTION;

    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }

    HANDLING READ;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE ma_analog_output_value:                                          */
/*                                                                            */
/*                Groesse des Signals am Analogausgang. Hier immer ein        */
/*                Stromausgang.                                               */
/* -------------------------------------------------------------------------- */

VARIABLE ma_analog_output_value
{
    LABEL [analog_output_value];
    HELP  [analog_output_help];

    CLASS ANALOG_OUTPUT & DYNAMIC;

    TYPE FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }

    CONSTANT_UNIT TXT_UNIT_MA;

    HANDLING READ & WRITE;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE ma_analog_output_alarm_select:  Fehlersignalisierung im          */
/*                                           Stoerungsfall.                   */
/* -------------------------------------------------------------------------- */

VARIABLE ma_analog_output_alarm_select
{
    LABEL TXT_AO_ALARM_CODE;
    HELP  [alarm_code_help];

    CLASS ANALOG_OUTPUT;

    TYPE ENUMERATED
    {
        {   0, [high]              },   /* uebersteuern                       */
        {   1, [low]               },   /* untersteuern                       */
        { 251, TXT_NO_SUPPLY       },   /* Wert weiterreichen                 */
        { 253, TXT_SPECIAL         }    /* halte letzten Wert oder Ersatzwert */
    }

    HANDLING READ;
}



/* ************************************************************************** */
/*                                                                            */
/*                       SV - Vergleichsstellentemperatur                     */
/*                                                                            */
/* ************************************************************************** */

COLLECTION OF VARIABLE reference_junction_temperature
{
    HELP  [electronics_temperature_help];
    LABEL [electronics_temperature];

    MEMBERS
    {
        /* NAME,        Variable                           ,Kurzbeschreibung,  Hilfstext */

        DIGITAL_UNITS,  electronics_temperature_digital_units, [units],        [digital_units_help];
        DIGITAL_VALUE,  electronics_temperature_digital_value, [blank],        [digital_value_help];
    }
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE electronics_temperature_digital_units:                           */
/*                                                                            */
/*                                  Einheit der Vergleichsstellentemperatur.  */
/* -------------------------------------------------------------------------- */

VARIABLE electronics_temperature_digital_units
{
    HELP  HLP_REFER_JUNC_UNIT;
    LABEL [cjc_temp_unit];

    CLASS INPUT & DYNAMIC;

    TYPE ENUMERATED
    {
        UNITS_CODE (32),              /* degrees Celsius */
        { 250, [not_used] }           /* not used        */
    }

    HANDLING READ;
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE electronics_temperature_digital_value:                           */
/*                                                                            */
/*                Groesse der Vergleichsstellentemperatur. Dieser Wert fuer   */
/*                interne und fuer eine externe Vergleichsstelle.             */
/* -------------------------------------------------------------------------- */

VARIABLE electronics_temperature_digital_value
{
    HELP  HLP_REFER_JUNC_VALUE;
    LABEL [cjc_temp_value]; 

    CLASS INPUT & DYNAMIC;

    TYPE FLOAT
    {
        MIN_VALUE -40.0;
        MAX_VALUE  85.0;

        DISPLAY_FORMAT  ".2f";      /* 2 Nachkommastellen,keine Mindestbreite */
        EDIT_FORMAT    "6.2f";
    }

    VALIDITY VALID_JUNCTMP          /* die Variable ist nur gueltig, wenn     */
                                    /* eine Vergleichsstelle parametriert ist */

    HANDLING READ & WRITE;
}



/* ************************************************************************** */
/*                                                                            */
/*                           TV - E I N G A N G   1                           */
/*                                                                            */
/* ************************************************************************** */

COLLECTION OF VARIABLE sensor1_measurement
{
    HELP  HLP_SENSOR1;
    LABEL [sensor_a_temperature_measurement_value];

    MEMBERS
    {
        DIGITAL_UNITS,        sensor1_digital_units,             [units],                                  HLP_SNSR_UNITS;
        DIGITAL_VALUE,        sensor1_measurement_digital_value, [sensor_a_temperature_measurement_value], HLP_SNSR1_VALUE;
    }
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE sensor1_digital_units:  Subset of Table II, Unit Codes           */
/*                                                                            */
/*                Elektrische Einheit an Eingang 1 des Feldgeraetes.          */
/* -------------------------------------------------------------------------- */

VARIABLE sensor1_digital_units
{
    HELP  HLP_SNSR_UNITS;
    LABEL [units];

    TYPE ENUMERATED
    {
        UNITS_CODE (36),                /* millivolts */
        UNITS_CODE (37),                /* ohms */
        { 250, "Not Used" }
    }

    HANDLING READ;

    CLASS INPUT & DYNAMIC;              /* DYNAMIC ist Bugfix fuer die nicht  */
                                        /* funktionierende REFRESH-Funktion.  */
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE sensor1_digital_value:  Elektrische Messgroesse am Eingang 1.    */
/*                                   Der Wert ist bereits durch die           */
/*                                   Abgleichparameter (z.B. Leitungs-        */
/*                                   abgleich) korrigiert.                    */
/* -------------------------------------------------------------------------- */

VARIABLE sensor1_measurement_digital_value
{
    HELP  HLP_SNSR1_VALUE;
    LABEL [sensor_a_temperature_measurement_value];

    CLASS INPUT & DYNAMIC;

    TYPE    FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }

    HANDLING READ;
}



/* ************************************************************************** */
/*                                                                            */
/*                           QV - E I N G A N G   2                           */
/*                                                                            */
/* ************************************************************************** */

COLLECTION OF VARIABLE sensor2_measurement
{
    HELP  HLP_SENSOR2;
    LABEL [sensor_b_temperature_measurement_value];

    MEMBERS
    {
        DIGITAL_UNITS,  sensor2_digital_units,             [units],                                  HLP_SNSR_UNITS;
        DIGITAL_VALUE,  sensor2_measurement_digital_value, [sensor_b_temperature_measurement_value], HLP_SNSR2_VALUE;
    }
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE sensor2_digital_units:  Subset of Table II, Unit Codes           */
/*                                                                            */
/*                Elektrische Einheit an Eingang 2 des Feldgeraetes.          */
/* -------------------------------------------------------------------------- */

VARIABLE sensor2_digital_units
{
    HELP  HLP_SNSR_UNITS;
    LABEL [units];

    TYPE ENUMERATED
    {
        UNITS_CODE (36),                /* millivolts */
        UNITS_CODE (37),                /* ohms */
        { 250, "Not Used" }
    }

    VALIDITY VALID_MIDIFF               /* Variable nur bei 2-kanaliger */
                                        /* Messung gueltig.             */
    HANDLING READ;

    CLASS INPUT & DYNAMIC;              /* DYNAMIC ist Bugfix fuer die nicht  */
                                        /* funktionierende REFRESH-Funktion.  */
}



/* -------------------------------------------------------------------------- */
/*  VARIABLE sensor2_digital_value:  Elektrische Messgroesse am Eingang 2.    */
/*                                   Der Wert ist bereits durch die           */
/*                                   Abgleichparameter (z.B. Leitungs-        */
/*                                   abgleich) korrigiert.                    */
/* -------------------------------------------------------------------------- */

VARIABLE sensor2_measurement_digital_value
{
    HELP  HLP_SNSR2_VALUE;
    LABEL [sensor_b_temperature_measurement_value];

    CLASS INPUT & DYNAMIC;

    TYPE    FLOAT
    {
        DISPLAY_FORMAT ".2f";
    }

    VALIDITY VALID_MIDIFF               /* Variable nur bei 2-kanaliger */
                                        /* Messung gueltig.             */
    HANDLING READ;
}

