/*##CODE-DATEI
;-------------------------------------------------------------------------------
; ##Copyright        :  ABB Automation Products GmbH
;
;-------------------------------------------------------------------------------
; ##System           :  CONTRANS T
; ##Systemkomponente :  Temperaturmessumformer TH02
;                       (Transmitter spez. Dokument: Rev. 1)
;-------------------------------------------------------------------------------
; ##Programm         :  HC-DDL
; ##Modul            :  TH02IM
; ##Pfad/Datei       :  TH02IM.DDL
; ##Library/Overlay  :  -/-
; ##Programmierer    :  K.Pose / PLC-Alz
; ##Erst.-Datum      :  02.03.2000
; ##Version/Datum    :  01.00 / 02.03.2000
;-------------------------------------------------------------------------------
; ##Beschreibung     :  Definitionsaenderung der folgenden Komponenten:
;
;                       - Common-Tables             Rev. 6.0
;                       - Universal Commands        Rev. 5.1
;                       - Common Practice Commands  Rev. 7.0
; ##SPECEND
;-------------------------------------------------------------------------------
; ###*/


#define DEVICE_BUSY              32


/*
 *  Importieren der brauchbaren Standard-HART-Tabellen...
 *
 *  Definition der Tabellen:  tables.ddl
 */

IMPORT STANDARD _TABLES, DEVICE_REVISION 6, DD_REVISION 1
{

    VARIABLE units_code;                       /* Tab. 2: Prozess-Einheiten   */
    VARIABLE transfer_function_code;           /* Tab. 3: Uebertragungsfkt.   */
    VARIABLE alarm_selection_code;             /* Tab. 6: Alarm-Codierung     */
    VARIABLE write_protect_code;               /* Tab. 7: Schreibschutz       */
    VARIABLE company_identification_code;      /* Tab. 8: Herstellertabelle   */
    VARIABLE flag_assignment;                  /* Tab.11: Flags (z.B. #39)    */
    VARIABLE operating_mode_code;              /* Tab.14: Operationsmodus     */

    EVERYTHING;

    REDEFINITIONS
	{

		VARIABLE company_identification_code
		{
		  REDEFINE TYPE ENUMERATED
		  {
			   /* Design Note: Leave 240-249 Undefined for future use. */
			   /* Design Note: Any enumerations in the range from 250-255
				  are 'Reserved' for a single definition if not defined. */
			   /* Design Note: Any enumeration not covered by another "Design
				  Note" is 'Undefined'. */
			   /* Design Note: The 'Unique Identifier' of the HART Frame
				  is composed of: (from most significant VARIABLE to least
				  significant VARIABLE: company_identification_code, *_model_code,
				  device_id) & (0x3FFFFFFFFF). Thus the meaning of the most
				  significant two bits of the 'Unique Identifier' are removed,
				  and thus are removed from the company_identification_code. This
				  then makes enumerations > 0x3F = 63 potentially not unique from
				  a HART Protocol viewpoint. To alleviate this problem a suggestion
				  should be made to the companies with enumerations > 63 to start
				  their allocation of *_model_code's from 249 backwards to 0. */


			 { 22,   [ABB] },
			 { 26,   [ABB] }
		  }
		}
	}

}



/*
 *  Import und - falls notwendig - Korrektur der durch das
 *  Universal-Command-Dokument festgelegten Daten.
 *
 *  Definition der Universal-Commands:  univ5.ddl
 */


IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 5, DD_REVISION 1
{
    EVERYTHING;


    REDEFINITIONS
    {

        VARIABLE manufacturer_id
        {
            /*
             *   Hersteller-Kodierung...
             */

            REDEFINE TYPE ENUMERATED
            {
                { 22,  [ABB] },
                { 26 , [ABB] }
            }
        }


        VARIABLE device_type
        {
            /*
             *   Geraete-Kodierung fuer H&B-Messumformer...
             */

            REDEFINE TYPE ENUMERATED
            {
                { 8,  TXT_DEV_TH02      }                /*  TH02/102/202     */
            }
        }


        /* Tab.10: phys. Schnittstelle */
        VARIABLE physical_signaling_code
        {
            REDEFINE TYPE ENUMERATED
            {
                { 0, [bell_202_current] },        /* FSK-stromueberlagert    */
                { 1, [bell_202_voltage] },        /* FSK-spannungsueberlagert */
                { 2, TXT_CMOS           },        /* Lokale Komm.-schnittst. */
                { 3, TXT_RS232          }         /* serielle RS232C         */
            }
        }



        /*  Kommando #0:  Kommunikation initialisieren...                     */
        COMMAND read_unique_identifier
        {
            /*
             *  auch bei Stoerungen soll eine Kommunikationsaufnahme
             *  moeglich sein, um die Diagnose zu ermoeglichen...
             */

            RESPONSE_CODES
            {
                DELETE 16;
            }
        }

        /*  Kommando #11:  Verbindungsaufbau ueber TAG...                     */
        COMMAND read_unique_identifier_with_tag
        {
            /*
             *  auch bei Stoerungen soll eine Kommunikationsaufnahme
             *  moeglich sein, um die Diagnose zu ermoeglichen...
             */

            RESPONSE_CODES
            {
                DELETE 5;
                DELETE 16;
            }
        }

		VARIABLE private_label_distributor
			{
			  REDEFINE TYPE ENUMERATED
			  {
				{ 22 , [ABB] },
				{ 26 , [ABB] }
			  }
			}

    }   /* end REDEFINITIONS */


}   /* end IMPORT_STANDARD_UNIVERSAL */






/*
 *  Import und - falls notwendig - Korrektur der durch das
 *  Common-Practice-Command-Dokument festgelegten Daten.
 *
 *  Definition der Commmon-Practice-Commands:  - common7.ddl
 *                                             - cmn_7.ddl
 */


#define ROM_CHECKSUM            0x01                 /* Kommando #48, Byte #0 */
#define RAM_FAILURE             0x02
#define EEPROM_WRITE            0x04
#define EEPROM_READ             0x08
#define EEPROM_WRITECYCLE       0x10
#define INVALID_EEPROM          0x20

#define ADC_NODATA              0x01                 /* Kommando #48, Byte #2 */

#define ABGLEICH_INAKTIV        0x01                 /* Kommando #48, Byte #3 */
#define EINGANG_ABGLEICH        0x02
#define EINGANG1_JUSTIERUNG     0x04
#define EINGANG2_JUSTIERUNG     0x08
#define PARAMETER_FEHLER        0x80

#define SENSOR1_BRUCH           0x01                 /* Kommando #48, Byte #4 */
#define SENSOR1_KURZ            0x02
#define SENSOR1_UNTER           0x04
#define SENSOR1_UEBER           0x08
#define SENSOR2_BRUCH           0x10
#define SENSOR2_KURZ            0x20
#define SENSOR2_UNTER           0x40
#define SENSOR2_UEBER           0x80

#define AO_UNTERSTEUERT         0x01                 /* Kommando #48, Byte #5 */
#define AO_UEBERSTEUERT         0x02
#define AO_ERSATZWERT           0x04
#define AO_LETZTER_WERT         0x08
#define AO_FIXED                0x10


IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 2
{

    /*
     *   die folgende Variablen, Kommandos und Methoden werden importiert...
     */

    COMMAND   write_pv_damping_value;                         /* Kommando #34 */
    COMMAND   write_pv_range_values;                          /* Kommando #35 */

    COMMAND   set_pv_upper_range_value;                       /* Kommando #36 */
    COMMAND   set_pv_lower_range_value;                       /* Kommando #37 */

    COMMAND   reset_configuration_change_flag;                /* Kommando #38 */

    VARIABLE  eeprom_select;                                  /* Kommando #39 */
    COMMAND   eeprom_control;

    COMMAND   enter_exit_fixed_pv_current_mode;               /* Kommando #40 */
    COMMAND   perform_transmitter_self_test;                  /* Kommando #41 */
    COMMAND   perform_master_reset;                           /* Kommando #42 */

    COMMAND   write_pv_units;                                 /* Kommando #44 */

    COMMAND   trim_pv_current_dac_zero;                       /* Kommando #45 */
    COMMAND   trim_pv_current_dac_gain;                       /* Kommando #46 */

    VARIABLE  xmtr_specific_status_0;                         /* Kommando #48 */
//    VARIABLE  xmtr_specific_status_1;
    VARIABLE  xmtr_specific_status_2;
    VARIABLE  xmtr_specific_status_3;
    VARIABLE  xmtr_specific_status_4;
    VARIABLE  xmtr_specific_status_5;

    COMMAND   read_additional_transmitter_status;

    COMMAND   write_pv_sensor_serial_number;                  /* Kommando #49 */

    VARIABLE  response_preambles;                             /* Kommando #59 */
    COMMAND   write_number_of_response_preambles;


    /*
     *  loop_test:  Der Analogausgang wird auf einen einstellbaren, konstanten
     *              Wert gesetzt. Im Unterschied zur Simulation wird dazu
     *              aufgefordert, eine nachgeschaltete Regelautomatik ausser
     *              Betrieb zu nehmen.
     */

    METHOD loop_test;


    /*
     *  dac_trim:   Abgleich des Analogausgangs an den festen Werten 4 und 20mA.
     *
     *              Anmerkung: - Sicherung der Daten ueber Kommando #39 fehlt.
     */

    METHOD dac_trim;


    /*
     *  scaled_dac_trim:  Abgleich des Analogausgangs. Auch hier wird der
     *              Ausgang mit den Werten 4 und 20mA abgeglichen. Das
     *              angeschlossene Messgeraet, z.B. ein Schalttafelinstrument,
     *              muss jedoch nicht auf "mA" geeicht sein, weil hier die
     *              Eingabe der Abgleichwerte in der Prozessgroesse (z.B. 0 und
     *              100%) erfolgen kann.
     *
     *              Anmerkung: - Die Funktion erlaubt NICHT den Abgleich bei
     *                           beliebigen Ausgangsstroemen.
     *                         - Sicherung der Daten ueber Kommando #39 fehlt.
     */

    METHOD scaled_dac_trim;


    /*
     *  leave_fixed_current_mode:  Der Konstantstrombetrieb des Analogausgangs
     *              wird abgeschaltet.
     */

    METHOD leave_fixed_current_mode;


    /*
     *  applied_rerange:  Die Methode ermoeglicht das Setzen von Messanfang und
     *              Messende durch das Anlegen der entsprechenden Prozess-
     *              werte an den Eingang des Messumformers.
     *
     *              Anmerkung: - Sicherung der Daten ueber Kommando #39 fehlt.
     */

    METHOD applied_rerange;


    /*
     *  return_to_normal:  Die Methode meldet, dass eine nachgeschaltete
     *              Regelung wieder aktiviert werden darf.
     *              Die Meldung muss vom Benutzer quittiert werden.
     */

    METHOD    return_to_normal;


    /*
     *  burn_eeprom:  Die Methode sichert die Daten netzausfallsicher, die
     *                durch Druecken der <Send>-Taste uebertragen werden.
     *                Die Methode wird dabei (undokumentiert) automatisch
     *                aufgerufen...
     */

    METHOD    burn_eeprom;


    REDEFINITIONS
    {

        /*  Kommando #34: Schreiben der Zeitkonstante fuer die Daempfung...   */
        COMMAND write_pv_damping_value
        {
            RESPONSE_CODES
            {
                DELETE 8;
            }
        }


        /*  Kommando #35: Schreiben des Messbereiches...                      */
        COMMAND write_pv_range_values
        {
            /*
             *  Response-Kode 14 geaendert: Die Warnung soll durch eine
             *                              Fehlermeldung ersetzt werden, weil
             *                              der Messumformer die Annahme der
             *                              Daten bei zu kleiner Spanne
             *                              verweigert.
             */

            REDEFINE RESPONSE_CODES
            {
                0,      SUCCESS,                [no_command_specific_errors];
                2,      DATA_ENTRY_ERROR,       [invalid_selection];
                5,      MISC_ERROR,             [too_few_data_bytes_recieved];
                6,      MISC_ERROR,             [xmtr_specific_command_error];
                7,      MODE_ERROR,             [in_write_protect_mode];
                9,      DATA_ENTRY_ERROR,       [lower_range_value_too_high];
                10,     DATA_ENTRY_ERROR,       [lower_range_value_too_low];
                11,     DATA_ENTRY_ERROR,       [upper_range_value_too_high];
                12,     DATA_ENTRY_ERROR,       [upper_range_value_too_low];
                13,     DATA_ENTRY_ERROR,       [URV_and_LRV_out_of_limits];
                14,     DATA_ENTRY_ERROR,       [span_too_small];
                16,     MODE_ERROR,             [access_restricted];
            }
        }


        /*  Kommando #41: Transmitter-Selbsttest durchfuehren...              */
        COMMAND perform_transmitter_self_test
        {
            RESPONSE_CODES
            {
                DELETE 16;      /*  zu Diagnosezwecken soll ein Selbsttest    */
                                /*  auch bei Stoerungen immer durchfuehrbar   */
                                /*  sein.                                     */
            }
        }


        /*  Kommando #42: Warmstart des Transmitters...                       */
        COMMAND perform_master_reset
        {
            RESPONSE_CODES
            {
                DELETE 16;      /*  ein Warmstart soll immer moeglich sein,   */
                                /*  damit der Messumformer auch nach einer    */
                                /*  Stoerung wieder auf den Normalbetrieb     */
                                /*  zurueckgestellt werden kann.              */
            }
        }



        /*  Kommando #48: Lesen weiterer Messumformer-Staties...              */

        REDEFINE VARIABLE xmtr_specific_status_0
        {
            HELP  HLP_SPEICHER_STATUS;
            LABEL TXT_SPEICHER_STATUS;

            HANDLING READ;
            CLASS DEVICE & DIAGNOSTIC & DYNAMIC;

            TYPE BIT_ENUMERATED
            {
                { ROM_CHECKSUM,        [rom_chksum_error],     HARDWARE },
                { RAM_FAILURE,         [ram_failure],          HARDWARE },
                { EEPROM_WRITE,        [eeprom_write_error],   HARDWARE },
                { EEPROM_READ,         TXT_EEPROM_READ,        HARDWARE },
                { EEPROM_WRITECYCLE,   [eeprom_write_error],   HARDWARE },
                { INVALID_EEPROM,      TXT_EEPROM_FAILURE,     HARDWARE }
            }
        }


        REDEFINE VARIABLE xmtr_specific_status_2
        {
            HELP  HLP_INPUT_STATUS;
            LABEL TXT_INPUT_STATUS;

            HANDLING READ;
            CLASS DEVICE & DIAGNOSTIC & DYNAMIC;

            TYPE BIT_ENUMERATED
            {
                { ADC_NODATA,          TXT_ADC_NODATA,         HARDWARE }
            }
        }


        REDEFINE VARIABLE xmtr_specific_status_3
        {
            HELP  HLP_OP_STATUS;
            LABEL TXT_OP_STATUS;

            HANDLING READ;
            CLASS DEVICE & DIAGNOSTIC & DYNAMIC;

            TYPE BIT_ENUMERATED
            {
                { ABGLEICH_INAKTIV,    TXT_OP_ABGLEICH,        MODE & AO1(AUTO & BAD) },
                { EINGANG_ABGLEICH,    TXT_OP_E_ABGLEICH,      MODE & AO1(AUTO & BAD) },
                { EINGANG1_JUSTIERUNG, TXT_OP_ABGL1_FAILURE,   MISC                   },
                { EINGANG2_JUSTIERUNG, TXT_OP_ABGL2_FAILURE,   MISC                   },
                { PARAMETER_FEHLER,    TXT_OP_PARA_FAILURE,    DATA                   }
            }
        }


        REDEFINE VARIABLE xmtr_specific_status_4
        {
            HELP  HLP_SENSOR_STATUS;
            LABEL TXT_SENSOR_STATUS;

            HANDLING READ;
            CLASS DEVICE & DIAGNOSTIC & DYNAMIC;

            TYPE BIT_ENUMERATED
            {
                { SENSOR1_BRUCH,       TXT_SNSR1_BRUCH,        HARDWARE },
                { SENSOR1_KURZ,        TXT_SNSR1_KURZ,         HARDWARE },
                { SENSOR1_UNTER,       TXT_SNSR1_UNTER,        PROCESS  },
                { SENSOR1_UEBER,       TXT_SNSR1_UEBER,        PROCESS  },
                { SENSOR2_BRUCH,       TXT_SNSR2_BRUCH,        HARDWARE },
                { SENSOR2_KURZ,        TXT_SNSR2_KURZ,         HARDWARE },
                { SENSOR2_UNTER,       TXT_SNSR2_UNTER,        PROCESS  },
                { SENSOR2_UEBER,       TXT_SNSR2_UEBER,        PROCESS  }
            }
        }


        REDEFINE VARIABLE xmtr_specific_status_5
        {
            HELP  HLP_OUTPUT_STATUS;
            LABEL TXT_OUTPUT_STATUS;

            HANDLING READ;
            CLASS DEVICE & DIAGNOSTIC & DYNAMIC;

            TYPE BIT_ENUMERATED
            {
                { AO_UNTERSTEUERT,     TXT_AO_UNTERSTEUERT,    MISC     },
                { AO_UEBERSTEUERT,     TXT_AO_UEBERSTEUERT,    MISC     },
                { AO_ERSATZWERT,       TXT_AO_ERSATZWERT,      MISC     },
                { AO_LETZTER_WERT,     TXT_AO_LETZTER_WERT,    MISC     },
                { AO_FIXED,            TXT_AO_FIXED,           MISC     }
            }
        }



        /*
         *   Kommando 48 neu definieren
         */

        REDEFINE COMMAND read_additional_transmitter_status
        {
            NUMBER 48;

            OPERATION READ;

            TRANSACTION
            {
                REQUEST
                {
                }

                REPLY
                {
                    /*
                     *  Die Statusbytes mit dem Wert 0x00 (1 Byte) werden beim
                     *  TEU211/TS11 nicht unterstuetzt (z.B. Relais, Led's).
                     */

                    response_code,
                    device_status,

                    xmtr_specific_status_0,
                    0,                    /* damit der Schreibzyklenzaehler   */
                                          /* nicht staendig angezeigt wird.   */
                    xmtr_specific_status_2,
                    xmtr_specific_status_3,
                    xmtr_specific_status_4,
                    xmtr_specific_status_5,
                    0 /* operation_mode_1 */,
                    0 /* operation_mode_2 */,
                    0, 0, 0,
                    0, 0, 0,
                    messart,
                    wire_no,
                    n_sensor,
                    sensor_typ,
                    sensor_monitor,
                    alter_selector,
                    usr_threshold_status,
                    0,
                    0
                }
            }

            RESPONSE_CODES
            {
                0,   SUCCESS,       [no_command_specific_errors];
                6,   MODE_ERROR,    [xmtr_specific_command_error];
                8,   MISC_WARNING,  TXT_STATUS_UPDATE;
            }
        }



        REDEFINE METHOD dac_trim
        {
            LABEL [DAC_trim];
            HELP  [digital_to_analog_trim_help];

            /*
             *  APPHELP "This type of calibration is usually used to compensate"
             *          " for drifting or aging output electronics";
             */

            DEFINITION
            {
                int  retry;
                int  flags;
                int  selection;
                char status[STATUS_SIZE];
                long var_ids[2];
                char disp_string1[60];
                char disp_string2[60];
                int  slen1;
                int  slen2;

                slen1=60;
                slen2=60;

                /*  Automatische Prozess-Steuerung ausschalten...  */
                get_dictionary_string( remove_loop_from_auto,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );
                add_abort_method( return_to_normal );

                /*  genaues Messgeraet an der Ausgang anschliessen... */
                get_dictionary_string( connect_reference_meter,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );

                IGNORE_ALL_DEVICE_STATUS();
                IGNORE_ALL_RESPONSE_CODES();
                XMTR_IGNORE_ALL_DEVICE_STATUS();
                XMTR_IGNORE_ALL_RESPONSE_CODES();

                /*
                 *  1.:  Abgleich bei 4mA...
                 */
                get_dictionary_string( setting_fld_dev_4_ma,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );
                assign_float( PV.ANALOG_VALUE,4.0 );
                send( 40,status );

                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status( 40,status[STATUS_RESPONSE_CODE] );
                    get_dictionary_string( abort_last_error,disp_string1,slen1 );
                    DELAY( 2,disp_string1 );
                    process_abort();
                }

                remove_abort_method( return_to_normal );
                add_abort_method( leave_fixed_current_mode );

                var_ids[0] = VARID( PV.ANALOG_VALUE );

                do
                {
                    get_dictionary_string( enter_meter_value,disp_string1,slen1 );
                    GET_DEV_VAR_VALUE( disp_string1,PV.ANALOG_VALUE );
                    send( 45,status );
                    assign_float( PV.ANALOG_VALUE,4.0 );
                    if( (status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4) )
                    {
                        display_response_status( 45,status[STATUS_RESPONSE_CODE] );
                        retry = 1;
                    }
                    else if( status[STATUS_RESPONSE_CODE] )
                    {
                        display_response_status( 45,status[STATUS_RESPONSE_CODE] );
                        get_dictionary_string( abort_last_error,disp_string1,slen1 );
                        DELAY( 2,disp_string1 );
                        process_abort();
                    }
                    else
                    {
                        get_dictionary_string( fld_dev_otpt_equal_ref_meter,disp_string1,slen1 );
                        get_dictionary_string( yes_no,disp_string2,slen2 );
                        retry=select_from_list( disp_string1,var_ids,disp_string2 );
                    }
                }
                while( retry );

                /*
                 *  2.:  Abgleich bei 20mA...
                 */
                get_dictionary_string( setting_fld_dev_20_ma,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );
                assign_float( PV.ANALOG_VALUE,20.0 );
                send( 40,status );

                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status( 40,status[STATUS_RESPONSE_CODE] );
                    get_dictionary_string( abort_last_error,disp_string1,slen1 );
                    DELAY( 2,disp_string1 );
                    process_abort();
                }

                do
                {
                    get_dictionary_string( enter_meter_value,disp_string1,slen1 );
                    GET_DEV_VAR_VALUE( disp_string1,PV.ANALOG_VALUE );
                    send( 46,status );
                    assign_float( PV.ANALOG_VALUE,20.0 );

                    if( (status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4) )
                    {
                        display_response_status( 46,status[STATUS_RESPONSE_CODE] );
                        retry = 1;
                    }
                    else if( status[STATUS_RESPONSE_CODE] )
                    {
                        display_response_status( 46,status[STATUS_RESPONSE_CODE] );
                        get_dictionary_string( abort_last_error,disp_string1,slen1 );
                        DELAY( 2,disp_string1 );
                        process_abort();
                    }
                    else
                    {
                        get_dictionary_string( fld_dev_otpt_equal_ref_meter,disp_string1,slen1 );
                        get_dictionary_string( yes_no,disp_string2,slen2 );
                        retry=select_from_list( disp_string1,var_ids,disp_string2 );
                    }
                }
                while( retry );

                /* ---------------------------------------------------------- */
                /*  und jetzt noch netzausfallsicher speichern...             */
                /* ---------------------------------------------------------- */

                DELAY( 3,TXT_EEPROM_WAIT );                /* Bitte warten... */

                iassign(eeprom_select,0);                   /* EEPROM brennen */
                send(39,status);

                if( status[STATUS_RESPONSE_CODE] )
                {
                    /* Daten konnten nicht gebrannt werden */
                    display_response_status(39,status[STATUS_RESPONSE_CODE]);
                    ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
                }

                process_abort() ;
            }
        }


        REDEFINE METHOD scaled_dac_trim
        {
            LABEL [scaled_DAC_trim];
            HELP  [digital_to_analog_scaled_trim_help];

            /*
             *  APPHELP "This type of calibration is usually used to compensate"
             *          " for drifting or aging output electronics. A common"
             *          " usage of this is when the reference is in different"
             *          " engineering units than the Analog Output"
             *          " (Ex. reference meter is 0-100% when the Field Device"
             *          " output is in mA";
             */

            DEFINITION
            {
                int   retry;
                int   flags;
                int   selection;
                char  status[STATUS_SIZE];
                float low, high, tmp_av;
                char  disp_string1[60];
                char  disp_string2[60];
                int   slen1;
                int   slen2;

                slen1 = 60;
                slen2 = 60;

                IGNORE_ALL_DEVICE_STATUS();
                XMTR_IGNORE_ALL_DEVICE_STATUS();
                IGNORE_ALL_RESPONSE_CODES();
                XMTR_IGNORE_ALL_RESPONSE_CODES();

                low  = 4.0;
                high = 20.0;

                get_dictionary_string( remove_loop_from_auto,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );
                add_abort_method( return_to_normal );

                do
                {
                    get_dictionary_string( trim_scaled_from_low_to_high,disp_string1,slen1 );
                    get_dictionary_string( proceed_change,disp_string2,slen2 );
                    retry = SELECT_FROM_LIST( disp_string1,disp_string2 );
                    if( retry )
                    {
                        do
                        {
                            get_dictionary_string( set_scale_lo_output_value,disp_string1,slen1 );
                            GET_LOCAL_VAR_VALUE(disp_string1, low);
                            get_dictionary_string( set_scale_hi_output_value,disp_string1,slen1 );
                            GET_LOCAL_VAR_VALUE(disp_string1, high);
                            if( !(high - low) )
                            {
                                get_dictionary_string( high_low_0_invalid,disp_string1,slen1 );
                                ACKNOWLEDGE( disp_string1 );
                            }
                        } while( !(high - low) );
                    }
                } while( retry );

                get_dictionary_string( connect_reference_meter,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );

                get_dictionary_string( setting_fld_dev_4_ma,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );
                assign_float( PV.ANALOG_VALUE,4.0 );
                send( 40,status );
                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status( 40,status[STATUS_RESPONSE_CODE] );
                    get_dictionary_string( abort_last_error,disp_string1,slen1 );
                    DELAY( 2,disp_string1 );
                    process_abort();
                }

                remove_abort_method( return_to_normal );
                add_abort_method( leave_fixed_current_mode );

                do
                {
                    tmp_av = low;
                    get_dictionary_string( enter_meter_value,disp_string1,slen1 );
                    GET_LOCAL_VAR_VALUE(disp_string1, tmp_av);
                    assign_float( PV.ANALOG_VALUE,16 / (high - low) * (tmp_av - low) + 4 );
                    send( 45,status );
                    if( (status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4) )
                    {
                        display_response_status( 45,status[STATUS_RESPONSE_CODE] );
                        retry = 1;
                    }
                    else if( status[STATUS_RESPONSE_CODE] )
                    {
                        display_response_status( 45,status[STATUS_RESPONSE_CODE] );
                        get_dictionary_string( abort_last_error,disp_string1,slen1 );
                        DELAY( 2,disp_string1 );
                        process_abort();
                    }
                    else
                    {
                        get_dictionary_string( scaled_otpt_low_eq_dev,disp_string1,slen1 );
                        get_dictionary_string( yes_no,disp_string2,slen2 );
                        retry=SELECT_FROM_LIST( disp_string1,disp_string2 );
                    }
                }
                while( retry );

                get_dictionary_string( setting_fld_dev_20_ma,disp_string1,slen1 );
                ACKNOWLEDGE( disp_string1 );
                assign_float( PV.ANALOG_VALUE,20.0 );
                send( 40,status );
                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status( 40,status[STATUS_RESPONSE_CODE] );
                    get_dictionary_string( abort_last_error,disp_string1,slen1 );
                    DELAY( 2,disp_string1 );
                    process_abort();
                }

                do
                {
                    tmp_av = high;
                    get_dictionary_string( enter_meter_value,disp_string1,slen1 );
                    GET_LOCAL_VAR_VALUE(disp_string1, tmp_av);
                    assign_float( PV.ANALOG_VALUE,16 / (high - low) * (tmp_av - low) + 4 );
                    send( 46,status );
                    if( (status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4) )
                    {
                        display_response_status( 46,status[STATUS_RESPONSE_CODE] );
                        retry = 1;
                    }
                    else if( status[STATUS_RESPONSE_CODE] )
                    {
                        display_response_status( 46,status[STATUS_RESPONSE_CODE] );
                        get_dictionary_string( abort_last_error,disp_string1,slen1 );
                        DELAY( 2,disp_string1 );
                        process_abort();
                    }
                    else
                    {
                        get_dictionary_string( scaled_otpt_high_eq_dev,disp_string1,slen1 );
                        get_dictionary_string( yes_no,disp_string2,slen2 );
                        retry=SELECT_FROM_LIST( disp_string1,disp_string2 );
                    }
                }
                while( retry );

                /* ---------------------------------------------------------- */
                /*  und jetzt noch netzausfallsicher speichern...             */
                /* ---------------------------------------------------------- */

                DELAY( 3,TXT_EEPROM_WAIT );                /* Bitte warten... */

                iassign(eeprom_select,0);                   /* EEPROM brennen */
                send(39,status);

                if( status[STATUS_RESPONSE_CODE] )
                {
                    /* Daten konnten nicht gebrannt werden */
                    display_response_status(39,status[STATUS_RESPONSE_CODE]);
                    ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
                }

                process_abort();
            }
        }


        REDEFINE METHOD applied_rerange
        {
            LABEL [apply_values];
            HELP  [applied_values_help];

            DEFINITION
            {
                int  rc;
                int  flags;
                int  selection;
                int  counter;
                long varid[3];
                char status     [STATUS_SIZE];
                char xmtr_status[MAX_XMTR_STATUS_LEN];
                char disp_string1[60];
                char disp_string2[60];
                int  slen1;
                int  slen2;

                slen1 =60;
                slen2 =60;

                IGNORE_ALL_RESPONSE_CODES();
                IGNORE_ALL_DEVICE_STATUS();
                XMTR_IGNORE_ALL_RESPONSE_CODES();
                XMTR_IGNORE_ALL_DEVICE_STATUS();

                ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );
                add_abort_method( return_to_normal );

                varid[0] = VARID( PV.DIGITAL_VALUE );
                varid[1] = VARID( PV.DIGITAL_UNITS );

                counter = 0;

                while( 1 )
                {
                    get_dictionary_string( set_the,disp_string1,slen1 );
                    get_dictionary_string( choice_4_20_exit,disp_string2,slen2 );

                    switch( SELECT_FROM_LIST( disp_string1,disp_string2 ) )
                    {
                        case 0:
                            /*  setze Messanfang (4mA) */
                            ACKNOWLEDGE( TXT_APPLY_4MA_INPUT );
                            rc = 2;
                            do
                            {
                                /* liegt ein sinnvoller Wert an ??? */
                                get_more_status(status,xmtr_status);
                                if( status[STATUS_RESPONSE_CODE] )
                                {
                                    display_response_status( 48,status[STATUS_RESPONSE_CODE] );
                                    break;
                                }
                                else
                                {
                                    /* Sensorstatus konnte abgefragt werden */
                                    if( xmtr_status[4] )
                                        ACKNOWLEDGE( TXT_APPLIED_SENS_FAIL );
                                    else
                                    {
                                        send(1,status);
                                        if( status[STATUS_RESPONSE_CODE] )
                                        {
                                            display_response_status( 1,status[STATUS_RESPONSE_CODE] );
                                            break;
                                        }
                                        else
                                        {
                                            get_dictionary_string( current_applied_process_val,disp_string1,slen1 );
                                            get_dictionary_string( set_4ma_read_new_leave,disp_string2,slen2 );
                                            rc = select_from_list( disp_string1,varid,disp_string2 );
                                        }
                                    }
                                }
                            }
                            while( rc==1 );

                            if( rc == 0 )
                            {
                                send( 37,status );
                                if( status[STATUS_RESPONSE_CODE] )
                                {
                                    display_response_status( 37,status[STATUS_RESPONSE_CODE] );
                                }
                                else
                                    counter++;

                                send_command( 15 );
                            }
                            break;

                        case 1:
                            /*  setze Messende (20mA) */
                            ACKNOWLEDGE( TXT_APPLY_20MA_INPUT );
                            rc = 2;
                            do
                            {
                                /* liegt ein sinnvoller Wert an ??? */
                                get_more_status(status,xmtr_status);
                                if( status[STATUS_RESPONSE_CODE] )
                                {
                                    display_response_status( 48,status[STATUS_RESPONSE_CODE] );
                                    break;
                                }
                                else
                                {
                                    /* Sensorstatus konnte abgefragt werden */
                                    if( xmtr_status[4] )
                                        ACKNOWLEDGE( TXT_APPLIED_SENS_FAIL );
                                    else
                                    {
                                        send(1,status);
                                        if( status[STATUS_RESPONSE_CODE] )
                                        {
                                            display_response_status( 1,status[STATUS_RESPONSE_CODE] );
                                            break;
                                        }
                                        else
                                        {
                                            get_dictionary_string( current_applied_process_val,disp_string1,slen1 );
                                            get_dictionary_string( set_20ma_read_new_leave,disp_string2,slen2 );
                                            rc = select_from_list( disp_string1,varid,disp_string2 );
                                        }
                                    }
                                }
                            }
                            while( rc==1 );

                            if( rc == 0 )
                            {
                                send( 36,status );
                                if( status[STATUS_RESPONSE_CODE] )
                                {
                                    display_response_status( 36,status[STATUS_RESPONSE_CODE] );
                                }
                                else
                                    counter++;

                                send_command( 15 );
                            }
                            break;

                        case 2:
                            /* Verlassen */
                            if( counter > 0 )
                            {
                                /* ------------------------------------------ */
                                /*  netzausfallsicher speichern...            */
                                /* ------------------------------------------ */

                                DELAY( 2,TXT_EEPROM_WAIT );
                                                           /* Bitte warten... */
                                iassign(eeprom_select,0);   /* EEPROM brennen */
                                send(39,status);

                                if( status[STATUS_RESPONSE_CODE] )
                                {
                                    /* Daten konnten nicht gebrannt werden */
                                    display_response_status(39,status[STATUS_RESPONSE_CODE]);
                                    ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
                                }
                            }

                            process_abort();

                            break;
                    }
                }
            }
        }



        REDEFINE METHOD return_to_normal
        {
            LABEL [aborting_return_to_normal];
            DEFINITION
            {
                ACKNOWLEDGE( TXT_CONNECT_AUTO_LOOP );
            }
        }


        REDEFINE METHOD burn_eeprom
        {
            LABEL [burn_eeprom];
            HELP [burn_eeprom_help];

            DEFINITION
            {
                char status[3];
                char disp_string1[60];
                int  counter;
                int  slen1;

                slen1 =60;

                IGNORE_ALL_RESPONSE_CODES();
                XMTR_IGNORE_ALL_RESPONSE_CODES();
                IGNORE_ALL_DEVICE_STATUS();
                XMTR_IGNORE_ALL_DEVICE_STATUS();

                assign_int(eeprom_select,0);
                get_dictionary_string(making_changes_permanent,disp_string1,slen1);
                DELAY(3,disp_string1);
                send(39,status);
                if( status[STATUS_RESPONSE_CODE] )
                {
                    display_response_status(39,status[STATUS_RESPONSE_CODE]);
                    get_dictionary_string(previous_err_seen_brn_eeprom,disp_string1,slen1);
                    ACKNOWLEDGE(disp_string1);
                }
                else
                {
                    /*
                     *  bis hierher Standard, aber das RESET-Bit sollte noch
                     *  geloescht werden...
                     */

                    counter = 0;

                    do
                    {
                        /*
                         *  Kommando #48 ausfhren, bis eine
                         *  Antwort kommt...
                         */

                        send(48,status);
                    }
                    while( (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY) && (counter++ < 10) );
                }
            }
        }

    }   /*  end  REDEFINITIONS  */

}   /*  end  IMPORT COMMON-PRACTICE-COMMANDS  */

