/*##CODE-DATEI
;-------------------------------------------------------------------------------
;##Copyright        :  ABB Automation Products GmbH
;
;-------------------------------------------------------------------------------
;##System           :  CONTRANS T
;##Systemkomponente :  Temperaturmessumformer TH02
;                      (Transmitter spez. Dokument: Rev. 1)
;-------------------------------------------------------------------------------
;##Programm         :  HC-DDL
;##Modul            :  TH02TP
;##Pfad/Datei       :  TH02TP.DDL
;##Library/Overlay  :  -/-
;##Programmierer    :  K.Pose / PLC-Alz
;##Erst.-Datum      :  02.03.2000
;##Version/Datum    :  01.00 / 02.03.2000
;-------------------------------------------------------------------------------
;##Beschreibung     :  Eingangsabgleich- und Preset-Funktionen
;                      ACHTUNG: Laenderspezifische Texte noch hier
;
;##SPECEND
;-------------------------------------------------------------------------------
;###*/


/* --------------- VARIABLEN ----------------- */

VARIABLE trimin_unit
{
    LABEL "trimin_unit";
    CLASS CORRECTION;
    TYPE  ENUMERATED
    {
        UNITS_CODE(36),
        UNITS_CODE(37)
    }
    HANDLING READ & WRITE;
}

VARIABLE trimin_wire_no
{
    LABEL "trimin_wire_no";
    CLASS CORRECTION;
    TYPE  ENUMERATED
    {
        { 2 , [two_wires]   },
        { 3 , [three_wires] },
        { 4 , [four_wires]  }
    }
    HANDLING READ;
}

VARIABLE trimin_LoLim1
{
    LABEL "trimin_LoLim1";
    CLASS CORRECTION;
    TYPE FLOAT;
    HANDLING READ;
}

VARIABLE trimin_HiLim1
{
    LABEL "trimin_HiLim1";
    CLASS CORRECTION;
    TYPE FLOAT;
    HANDLING READ;
}
VARIABLE trimin_LoLim2
{
    LABEL "trimin_LoLim2";
    CLASS CORRECTION;
    TYPE FLOAT;
    HANDLING READ;
}
VARIABLE trimin_HiLim2
{
    LABEL "trimin_HiLim2";
    CLASS CORRECTION;
    TYPE FLOAT;
    HANDLING READ;
}

VARIABLE preset_ctrl
{
    LABEL "preset_ctrl";
    CLASS DEVICE;
    TYPE ENUMERATED
    {
        { 0 , "ROM_ALL_PRESET"  },
        { 1 , "MFG_CAL_RESTORE" }
    }
    HANDLING WRITE;
}

/* --------------------COMMANDS---------------------- */

COMMAND lese_trimin_limits
{
    NUMBER 162;

    OPERATION READ;

    TRANSACTION
    {
        REQUEST
        { }

        REPLY
        {
            response_code, device_status,
            trimin_wire_no, trimin_unit,
            trimin_LoLim1,
            trimin_HiLim1,
            trimin_LoLim2,
            trimin_HiLim2
        }
    }

    RESPONSE_CODES
    {
        0,   SUCCESS,                 [no_command_specific_errors];
        6,   MODE_ERROR,              [xmtr_specific_command_error];
    }
}

COMMAND preset_command
{
    NUMBER 141;

    OPERATION COMMAND;

    TRANSACTION
    {
        REQUEST
        { preset_ctrl }

        REPLY
        {
            response_code, device_status,
            preset_ctrl
        }
    }

    RESPONSE_CODES
    {
        0,   SUCCESS,                 [no_command_specific_errors];
        2,   DATA_ENTRY_ERROR,        TXT_INVALID_PRESETCODE;
        5,   MISC_ERROR,              [too_few_data_bytes_recieved];
        6,   MODE_ERROR,              [xmtr_specific_command_error];
        7,   MODE_ERROR,              [in_write_protect_mode];
    }
}




/*##METHODE input_trim
;-------------------------------------------------------------------------------
;#Name             : input_trim
;
;#Dateiname        : TH02TP.DDL
;-------------------------------------------------------------------------------
;#Beschreibung     : Eingangsabgleich
;
;                    Die Methode fuehrt einen Eingangsabgleich durch.
;                    Mit Kommando #162 werden die Justiergrenzen der momentanen
;                    Konfiguration ermittelt.
;
;
;#Besonderheiten   : -/-
;
;-------------------------------------------------------------------------------
;##LOGBEGIN
;
;  ##1
;  ##Programmierer  : K.Pose / ABB Automation Products
;  ##Version/Datum  : 01.00 / 18.02.2000
;  ##Aenderungen    : erstellt.
;
;##LOGEND
;-------------------------------------------------------------------------------
;#PSEUDOCODE:
;
;-------------------------------------------------------------------------------
;##*/

METHOD input_trim
{
    LABEL TXT_INPUT_TRIM;
    HELP  HLP_INPUT_TRIM;

    DEFINITION
    {
        long  ids[4];
        float ftmp;
        int   eingang;
        int   select;
        int   anzahl_der_versuche;
        int   counter;
        int   success;
        char  status     [STATUS_SIZE];
        char  xmtr_status[MAX_XMTR_STATUS_LEN];

        IGNORE_ALL_DEVICE_STATUS();          /* diese Ueberwachungen werden   */
        IGNORE_ALL_RESPONSE_CODES();         /* in Eigenregie durchgefuehrt,  */
        XMTR_IGNORE_ALL_DEVICE_STATUS();     /* weil die Methode sonst bei    */
        XMTR_IGNORE_ALL_RESPONSE_CODES();    /* Statusmeldungen               */
                                             /* abbricht.                     */

        ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );

        add_abort_method( return_to_normal );

        get_more_status(status,xmtr_status);
        if(  xmtr_status[3] & 0x0F)
        {
                /*      Zwischen Schritt 1 und 2,beschaeftigt oder Fehler
                 *      Also: Reset ( Cmd #42 ) um Bits zu loeschen und
                 *      einen definierten Startzustand zu erreichen
                 */
        
                ACKNOWLEDGE( TXT_RAM_DATA_LOSE );

                send( 42,status );                              /* Warmstart ausloesen */
                if( status[STATUS_RESPONSE_CODE] )
                {
                    /* der Messumformer wollte nicht... */
                    display_response_status( 42,status[STATUS_RESPONSE_CODE] );
                    ACKNOWLEDGE( TXT_NO_MASTER_TEST );
                    process_abort();
                    /* ENDE */
                }

                DELAY( 3,TXT_RESET_WAIT );                      /* und 3s warten...   */

                anzahl_der_versuche = 0;
                do
                {
                    /* Kommando #48 sooft ausfuehren,    */
                    /* bis der Warmstart beendet ist... */
                    get_more_status(status,xmtr_status);
                }
                while(    (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY  )
                       && (anzahl_der_versuche++ < MAXIMUM_SELFTEST_TIME) );
        
                if( status[STATUS_RESPONSE_CODE] == DEVICE_BUSY )
                {
                    /* Geraet ist im Neustart blockiert. */
                    ACKNOWLEDGE( TXT_RESET_BLOCKED );
                    process_abort();
                    /* ENDE */
                }
                else if( status[STATUS_RESPONSE_CODE] )
                {
                    /* der Messumformer wollte nicht... */
                    display_response_status( 48,status[STATUS_RESPONSE_CODE] );
                    ACKNOWLEDGE( TXT_NO_MASTER_TEST );
                    process_abort();
                    /* ENDE */
                }

        } /* RESET fertig */

        /*      
         *      Jetzt gehts erst richtig los
         */

        eingang = ivar_value( messart );

        if( eingang & 0x08 )
        {
            /*
             *  mehrkanalige Messung; daher muss der Kanal zunaechst
             *  festgelegt werden...
             */

            select = SELECT_FROM_LIST( TXT_TRIMIN_SELK,TXT_TRIMIN_SELKO );
            if( select==0 )
            {
                ACKNOWLEDGE( TXT_TRIMIN_SHORT2 );
            }
            else
            {
                ACKNOWLEDGE( TXT_TRIMIN_SHORT1 );
            }
        }
        else
        {
            /*
             *  bei einkanaliger Messung ist immer der erste Kanal,
             *  d.h. der Eingang, gefragt...
             */
            select = 0;
        }
        
        send(162, status);

        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 162,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_TRIMIN );
            process_abort();
            /* ENDE */
        }
        
        ids[0]=VARID(trimin_unit);
        ids[1]=VARID(trimin_wire_no);

        ids[2]=VARID(trimin_LoLim1);
        ids[3]=VARID(trimin_HiLim1);

        if( select==0 )
        {
            acknowledge( VXT_TRIMC_PROMPT1, ids);
            acknowledge( VXT_TRIMSPR_PROMPT, ids);
            DELAY_TIME( 2 );
            send_command(3);
            ftmp=fvar_value(sensor1_measurement_digital_value);
        }
        else
        {
            acknowledge( VXT_TRIMC_PROMPT2, ids);
            acknowledge( VXT_TRIMSPR_PROMPT, ids);
            DELAY_TIME( 2 );
            send_command(3);
            ftmp=fvar_value(sensor2_measurement_digital_value);
        }

        if (( ftmp >= fvar_value(trimin_LoLim1)) && ( ftmp <= fvar_value(trimin_HiLim1)) )
        {
            fassign( sollwert, ftmp );
        }
        else
        {
            vassign( sollwert, trimin_LoLim1);
        }

        do
        {
            GET_DEV_VAR_VALUE( TXT_TRIMA1_PROMPT, sollwert );
        }
        while( fvar_value( sollwert ) < fvar_value(trimin_LoLim1) || fvar_value( sollwert ) > fvar_value( trimin_HiLim1 ));

        DELAY( 3,TXT_TRIMIN_WAIT );           /* Wartezeit, damit sich der    */
                                              /* Eingangswert beruhigen       */
                                              /* kann...                      */

        iassign     ( kanal,select+1 );       /* festgelegter Kanal im MU...  */
        send( 163,status );                   /* Eingangsabgleich starten...  */

        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 163,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_TRIMIN );
            process_abort();
            /* ENDE */
        }

        /* ------------------------------------------------------------------ */
        /*  Eingangsabgleich Schritt 1 gestartet ; jetzt muss gefragt         */
        /*  werden bis zum bitteren Ende. Anschliessend koennen die Fehler    */
        /*  gemeldet werden, wenn welche aufgetreten sind.                    */
        /* ------------------------------------------------------------------ */

        do
        {
            get_more_status(status,xmtr_status);       /* #48 zum Messumformer */
        }
        while( xmtr_status[3] & 0x02 );                /* Abgleich noch aktiv */

        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 48,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_TRIMIN_NO_SUCCESS );
        }
        else
        {
            if(    (select == 0 && xmtr_status[3] & 0x04)
                || (select == 1 && xmtr_status[3] & 0x08)
                ||  !(xmtr_status[3] & 0x01) )
            {
                /* beim Abgleich ist ein Fehler aufgetreten... */
                ACKNOWLEDGE( TXT_TRIMIN_NO_SUCCESS );   /* melde Fehler und    */
                ACKNOWLEDGE( TXT_TRIMIN_TIP_1 );        /* gebe Tips...        */
            }
            else
            {
                /* Jetzt kommt der 2. Schritt */

                ids[2]=VARID(trimin_LoLim2);
                ids[3]=VARID(trimin_HiLim2);

                acknowledge( VXT_TRIMSPR_PROMPT, ids);
                DELAY_TIME(2);
                send_command(3);
                if( select==0 )
                {
                    ftmp=fvar_value(sensor1_measurement_digital_value);
                }
                else
                {
                    ftmp=fvar_value(sensor2_measurement_digital_value);
                }

                if (( ftmp >= fvar_value(trimin_LoLim2)) && ( ftmp <= fvar_value(trimin_HiLim2)) )
                {
                    fassign( sollwert, ftmp );
                }
                else
                {
                    vassign( sollwert, trimin_HiLim2);
                }

                do
                {
                    GET_DEV_VAR_VALUE( TXT_TRIMA2_PROMPT, sollwert );
                }
                while( fvar_value(sollwert) < fvar_value(trimin_LoLim2) || fvar_value(sollwert) > fvar_value( trimin_HiLim2 ));

                DELAY( 3,TXT_TRIMIN_WAIT );            /* Wartezeit, damit sich der    */
                                                      /* Eingangswert beruhigen       */
                                                      /* kann...                      */

                iassign     ( kanal,select+1 );       /* festgelegter Kanal im MU...  */
                send( 163,status );                   /* Eingangsabgleich starten...  */

                if( status[STATUS_RESPONSE_CODE] )
                {
                    /* der Messumformer wollte nicht... */
                    display_response_status( 163,status[STATUS_RESPONSE_CODE] );
                    ACKNOWLEDGE( TXT_NO_TRIMIN );
                    process_abort();
                    /* ENDE */
                }
        
                /* ------------------------------------------------------------------ */
                /*  Eingangsabgleich Schritt 2 gestartet ; jetzt muss gefragt         */
                /*  werden bis zum bitteren Ende. Anschliessend koennen die Fehler    */
                /*  gemeldet werden, wenn welche aufgetreten sind.                    */
                /* ------------------------------------------------------------------ */
        
                do
                {
                    get_more_status(status,xmtr_status);       /* #48 zum Messumformer */
                }
                while( xmtr_status[3] & 0x02 );                /* Abgleich noch aktiv */
        
                if( status[STATUS_RESPONSE_CODE] )
                {
                    /* der Messumformer wollte nicht... */
                    display_response_status( 48,status[STATUS_RESPONSE_CODE] );
                    ACKNOWLEDGE( TXT_TRIMIN_NO_SUCCESS );
                }
                else
                {
                    if(    (select == 0 && xmtr_status[3] & 0x04)
                        || (select == 1 && xmtr_status[3] & 0x08) )
                    {
                        /* beim Abgleich ist ein Fehler aufgetreten... */
                        ACKNOWLEDGE( TXT_TRIMIN_NO_SUCCESS );   /* melde Fehler und    */
                        ACKNOWLEDGE( TXT_TRIMIN_TIP_1 );        /* gebe Tips...        */
                        process_abort();
                    }
                        

                    /* ---------------------------------------------------------- */
                    /*  Abgleich war erfolgreich, nun muessen die Daten noch       */
                    /*  netzausfallsicher gespeichert werden, damit nicht alles   */
                    /*  fuer die Katz' war...                                      */
                    /* ---------------------------------------------------------- */

                    DELAY( 2,TXT_EEPROM_WAIT );                /* Bitte warten... */

                    iassign(eeprom_select,0);                   /* EEPROM brennen */
                    send(39,status);

                    if( status[STATUS_RESPONSE_CODE] )
                    {
                        /* Daten konnten nicht gebrannt werden */
                        display_response_status(39,status[STATUS_RESPONSE_CODE]);
                        ACKNOWLEDGE( TXT_EEPROM_NO_SAVE );
                    }
                    else
                    {
                        /*
                         *  bis hierher Standard, aber das RESET-Bit sollte noch
                         *  geloescht werden...
                         */

                        counter = 0;

                        do
                        {
                            /*
                             *  Kommando #48 ausfuehren, bis eine
                             *  Antwort kommt...
                             */

                            send(48,status);
                        }
                            while( (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY) && (counter++ < 10) );

                            ACKNOWLEDGE( TXT_TRIMIN_SUCCESS );      /* melde Erfolg... */

                    } /* Burn OK */
                } /* 2.Schritt 48_confirmed */
            } /* 2.Schritt starten */
        } /* 1.Schritt 48_confirmed */

        process_abort();            /* Regelkreis wieder in Betrieb nehmen... */

    }       /* endDEF */

}       /* endMETHOD */


/* METHODE : FACTORY-PRESET */

METHOD preset_all
{
    LABEL TXT_PRESET_ALL;
    HELP  HLP_PRESET_ALL;
    DEFINITION
    {
        char  status     [STATUS_SIZE];
        char  xmtr_status[MAX_XMTR_STATUS_LEN];

        IGNORE_ALL_DEVICE_STATUS();          /* diese Ueberwachungen werden   */
        IGNORE_ALL_RESPONSE_CODES();         /* in Eigenregie durchgefuehrt,  */
        XMTR_IGNORE_ALL_DEVICE_STATUS();     /* weil die Methode sonst bei    */
        XMTR_IGNORE_ALL_RESPONSE_CODES();    /* Statusmeldungen               */

        ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );
        ACKNOWLEDGE( TXT_LOSE_PRESET_ALL );
        iassign( preset_ctrl, 0 );

        DELAY( 3,TXT_RUN_PRESET_ALL );

        assign_float( PV.ANALOG_VALUE,4.0 );
        send( 40,status );

        if( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status( 40,status[STATUS_RESPONSE_CODE] );
        }

        send(141, status);
        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 141,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_PRESET_ALL );
            process_abort();
            /* ENDE */
        }
        add_abort_method( preset_all_abort );
        DELAY_TIME(3);

/* Keine Ahnung,wie ich einer Ascii Variablen einen Konstanten Wert zuweisen soll

        ???assign( sensor_name, "PT 100 (IEC751)     " );
        send_trans( 136, 1, status );
        if( status[STATUS_RESPONSE_CODE] )
        {
            display_response_status( 136,status[STATUS_RESPONSE_CODE] );
        }

Keine Ahnung,Ende */

        process_abort();
    }
}

METHOD preset_all_abort
{
    LABEL "preset_all_abort";
    DEFINITION
    {
        int   anzahl_der_versuche;
        char  status     [STATUS_SIZE];
        char  xmtr_status[MAX_XMTR_STATUS_LEN];

        IGNORE_ALL_DEVICE_STATUS();          /* diese Ueberwachungen werden   */
        IGNORE_ALL_RESPONSE_CODES();         /* in Eigenregie durchgefuehrt,  */
        XMTR_IGNORE_ALL_DEVICE_STATUS();     /* weil die Methode sonst bei    */
        XMTR_IGNORE_ALL_RESPONSE_CODES();    /* Statusmeldungen               */

        iassign( preset_ctrl, 1 );
        PUT_MESSAGE( TXT_RUN_PRESET_MFGTRIM );

        send(141, status);
        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 141,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_ER_PRESET_MFGTRIM );
            process_abort();
            /* ENDE */
        }

        /* RESET = Parameter werden zurueckgesetzt
         * um ev. Kanal2-Statusbits zu loeschen
         */

        send( 42,status );                              /* Warmstart ausloesen */
        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 42,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_MASTER_TEST );
            process_abort();
            /* ENDE */
        }

        DELAY( 3,TXT_RESET_WAIT );                      /* und 3s warten...   */

        anzahl_der_versuche = 0;
        do
        {
            /* Kommando #48 sooft ausfuehren,    */
            /* bis der Warmstart beendet ist... */
            get_more_status(status,xmtr_status);
        }
        while(    (status[STATUS_RESPONSE_CODE] == DEVICE_BUSY  )
                       && (anzahl_der_versuche++ < MAXIMUM_SELFTEST_TIME) );
        
        if( status[STATUS_RESPONSE_CODE] == DEVICE_BUSY )
        {
            /* Geraet ist im Neustart blockiert. */
            ACKNOWLEDGE( TXT_RESET_BLOCKED );
            process_abort();
            /* ENDE */
        }
        else if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 48,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_MASTER_TEST );
            process_abort();
            /* ENDE */
        }

        /*
         *  jetzt muessen die ganzen statischen Daten geladen werden,
         *  weil die REFRESH-Funktion (ueberraschend?) nicht
         *  funktioniert...
         */

        send(14,status);
        if( status[STATUS_RESPONSE_CODE] )
            display_response_status(14,status[STATUS_RESPONSE_CODE]);

        send(15,status);
        if( status[STATUS_RESPONSE_CODE] )
            display_response_status(15,status[STATUS_RESPONSE_CODE]);

        send(16,status);
        if( status[STATUS_RESPONSE_CODE] )
            display_response_status(16,status[STATUS_RESPONSE_CODE]);

        send(48,status);
        if( status[STATUS_RESPONSE_CODE] )
            display_response_status(48,status[STATUS_RESPONSE_CODE]);

        send(132,status);
        if( status[STATUS_RESPONSE_CODE] )
            display_response_status(132,status[STATUS_RESPONSE_CODE]);

        ACKNOWLEDGE( TXT_PRESET_ALL_SUCCESS );
        process_abort();
    }
}


METHOD preset_mfgtrim
{
    LABEL TXT_PRESET_MFGTRIM;
    HELP  HLP_PRESET_MFGTRIM;
    DEFINITION
    {
        char  status     [STATUS_SIZE];
        char  xmtr_status[MAX_XMTR_STATUS_LEN];

        IGNORE_ALL_DEVICE_STATUS();          /* diese Ueberwachungen werden   */
        IGNORE_ALL_RESPONSE_CODES();         /* in Eigenregie durchgefuehrt,  */
        XMTR_IGNORE_ALL_DEVICE_STATUS();     /* weil die Methode sonst bei    */
        XMTR_IGNORE_ALL_RESPONSE_CODES();    /* Statusmeldungen               */

        ACKNOWLEDGE( TXT_REMOVE_AUTO_LOOP );
        ACKNOWLEDGE( TXT_LOSE_PRESET_MFGTRIM );
        add_abort_method( return_to_normal );
        iassign( preset_ctrl, 1 );

        DELAY( 3,TXT_RUN_PRESET_MFGTRIM );

        send(141, status);
        if( status[STATUS_RESPONSE_CODE] )
        {
            /* der Messumformer wollte nicht... */
            display_response_status( 141,status[STATUS_RESPONSE_CODE] );
            ACKNOWLEDGE( TXT_NO_PRESET_MFGTRIM );
            process_abort();
            /* ENDE */
        }
        ACKNOWLEDGE( TXT_PRESET_MFGTRIM_SUCCESS );        
        process_abort();
    }
}


